
#ifndef __PLUGIN_AVATAR_H__
#define __PLUGIN_AVATAR_H__

#include <windows.h>
#include "plugin_defs.h"

/**
 * WPARAM
 */
#define TLEN_AVATAR_CALLFUNCTION  "Tlen/Avatar/CallFunction"

/**
 * Pobranie wskaznika do obiektu awatara (avatarHandle).
 * LPARAM Wskaznik do struktury 'TlenAvatarDef',
 *        nalezy wypelnic protocolID, userID, avatarType.
 *
 * Uwaga: zwraca tymczasowa wartosc, mozna ja skopiowac
 *        za pomoca 'TLEN_AVATAR_CLONE_HANDLE'.
 */
#define TLEN_AVATAR_GET              1

/**
 * Ustawienie awatara.
 * LPARAM Wskaznik do struktury 'TlenAvatarDef',
 *        nalezy wypelnic protocolID, userID, avatarType,
 *        imagePath lub imageData, imageDataLen, imageType,
 *        oraz imageName.
 */
#define TLEN_AVATAR_SET              2

/**
 * Usuniecie awatara.
 * LPARAM Wskaznik do struktury 'TlenAvatarDef',
 *        nalezy wypelnic protocolID, userID, avatarType.
 */
#define TLEN_AVATAR_DEL              3

/**
 * Narysowanie awatara.
 * LPARAM Wskaznik do struktury 'TlenAvatarDef',
 *        nalezy wypelnic 'protocolID', 'userID', 'avatarType'
 *        lub 'avatarHandle', oraz 'hdc',
 *        do pozycjonowania awatara wypelnic 'frame'.
 */
#define TLEN_AVATAR_DRAW             4

/**
 * Pokazanie okienka wyboru awatara.
 * LPARAM Wskaznik do struktury 'TlenAvatarDef',
 *        nalezy wypelnic protocolID, userID i avatarType.
 */
#define TLEN_AVATAR_SHOW_FRAME       5

/**
 * Pobranie wartosci opcji.
 * LPARAM Rodzaj opcji (TlenAvatarOptions).
 */
#define TLEN_AVATAR_GET_OPTION       6


/**
 * Pobiera avatar z serwera, wywolywana jest funkcja 'PF_GETAVATAR'
 * dla danego protokolu,
 * plus odswiezane sa awatary w aplikacji.
 *
 * LPARAM Wskaznik do struktury 'TlenAvatarDef'.
 *        
 * Jesli uda sie pobrac awatar to jego handle 
 * pojawi sie w powiadomieniu. Bedzie to wartosc 
 * tymczasowa, wiec trzeba ja sobie skopjowac do 
 * dalszego przetwarzania.
 */
#define TLEN_AVATAR_GET_SERV         7

/**
 * Ustawienie avatar na serwerze, wywolywana jest funkcja 'PF_SETOWNAVATAR'
 * dla danego protokolu,
 * plus odswiezane sa awatary w aplikacji.
 *
 * LPARAM Wskaznik do struktury 'TlenAvatarDef'
 *
 * Jeslu uda sie ustawic awatar to nie bedzie 
 * tresci bledu w powiadomieniu.
 */
#define TLEN_AVATAR_SET_SERV         8

/**
 * Ustawienie avatar na serwerze, wywolywana jest funkcja 'PF_DELOWNAVATAR'
 * dla danego protokolu,
 * plus odswiezane sa awatary w aplikacji.
 *
 * LPARAM Wskaznik do struktury 'TlenAvatarDef'
 *
 * Jeslu uda sie usunac awatar to nie bedzie 
 * tresci bledu w powiadomieniu.
 */
#define TLEN_AVATAR_DEL_SERV         9

/**
 * Utworzenie obiektu awatara.
 * LPARAM Wskaznik do struktury 'TlenAvatarDef'.
 */
#define TLEN_AVATAR_CREATE_HANDLE    10

/**
 * Skopiowanie obiektu awatara.
 * LPARAM Handle awatara.
 */
#define TLEN_AVATAR_CLONE_HANDLE     11

/**
 * Zniszczenie obiektu awatara.
 * LPARAM Handle awatara.
 */
#define TLEN_AVATAR_DESTROY_HANDLE   12

/**
 * Rodzaje opcji dla awatarow.
 */
typedef enum
{
	TLEN_AVATAR_OPTIONS_NONE = 0,
	TLEN_AVATAR_OPTIONS_SHOW_IN_ROSTER,
	TLEN_AVATAR_OPTIONS_SHOW_ONLY_IF_SELECTED,
	TLEN_AVATAR_OPTIONS_SHOW_FOR_UNAVAILABLE,
	TLEN_AVATAR_OPTIONS_POSITION_IN_ROSTER,
	TLEN_AVATAR_OPTIONS_SHOW_IN_TOOLTIP,
	TLEN_AVATAR_OPTIONS_SHOW_IN_NOTIFICATION,
	TLEN_AVATAR_OPTIONS_PREFER_LOCAL

} TlenAvatarOptions;

/**
 * Typy awatarow.
 */
typedef enum
{
	TLEN_AVATAR_TYPE_PUBLIC = 0,
	TLEN_AVATAR_TYPE_LOCAL = 100
} TlenAvatarTypes;

/**
 * Struktura awatara.
 */
typedef struct
{
	/**
	 * Wielko struktury w bajtach
	 */
	int structSize;

	/**
	 * Identyfikator protokolu.
	 */
	char* protocolID;

	/**
	 * Identyfikator uzytkownika (np. login@tlen.pl, numer_gg)
	 */
	char* userID;

	/**
	 * Typ awatara.
	 */
	TlenAvatarTypes avatarType;

	/**
	 * Wskaznik do obiektu awatara.
	 */
	HANDLE avatarHandle;

	/**
	 * Sciezka do pliku obrazka.
	 */
	char* imagePath;

	/**
	 * Dane obrazka.
	 */
	unsigned char* imageData;

	/**
	 * Dlugosc danych obrazka.
	 */
	unsigned int imageDataLen;

	/**
	 * Typ obrazka.
	 */
	char *imageType;

	/**
	 * Nazwa, do wyswietlania w ostatnio uzywanych.
	 */
	char *imageName;

	/**
	 * Ramka kadrowania pliku lub pozycja i wielkosc
	 * awatara do narysowania,
	 * jesli right i bottom rowne 0 to rysowana jest
	 * aktualna wielkosc awatara.
	 */
	RECT frame;

	/**
	 * Kontekst urzadzenia do rysowania awatara.
	 */
	HDC hdc;

	/**
	 * Funkcja do otrzymywania powiadomien.
	 * wParam Wskaznik do struktury TlenAvatarDef.
	 * lParam Tresc bledu (char*).
	 */
	TLENFUNCTION callback;
	
	/**
	 * Dodatkowe dane;
	 */
	void *extra_data;

	/**
	 * Przy ustawianiu publicznego awatara,
	 * publiczny dla wszystkich kontaktow (1).
	 */
	int publicForAll;
	
} TlenAvatarDef;



#endif

