/* ==========================================================================
 * Name:        tlenek\contacticon.h
 * Purpose:     Deklaracja klasy ContactIcon reprezentujacej ikonke elementu
 *              na liscie kontaktow (ContactList)
 * Author:      Marcin 'malcom' Malich <me@malcom.pl>
 * Modified by:
 * Created:     21/09/2007 02:41:09
 * SVN-ID:      $Id: contacticon.h 14 2007-10-15 17:33:59Z malcom $
 * Copyright:   (c) 2007 Marcin 'malcom' Malich <me@malcom.pl>, Poland
 * Licence:     GNU Lesser General Public License
 * ========================================================================== */

#ifndef _TLENEK_CONTACT_ICON_H_
#define _TLENEK_CONTACT_ICON_H_

#include "tlenek\defs.h"
#include "tlenek\handler.h"
#include "tlenek\utils.h"
#include "tlen_sdk\plugin_contactlist_defs.h"


namespace tlenek {


class Image;


// rozmiar standardowej jednostki alokacji, dla tablicy ikonek
static const int ImageSizeAlloc = 10;


class TLENEK_DLLIMPEXP ContactIcon : private ContactListIconDef {

	private:

		int m_image_alloc;		// ilosc zaalokowanych ikonek w tablicy

		void Init();


	public:

		ContactIcon() {
			Init();
		}

		~ContactIcon();


		const char* GetId() const {
			return ID;
		}

		void SetId(const char* id) {
			my_cstrdup(ID, id);
		}


		int Count() const {
			return StatesCount;
		}


		enum ContactIconType {
			PRE = CONTACTLIST_ICONKIND_PRETEXT,
			POST = CONTACTLIST_ICONKIND_POSTTEXT,
		};


		ContactIconType GetType() const {
			return (ContactIconType) IconKind;
		}

		void SetType(ContactIconType type) {
			IconKind = type;
			if (structSize == 0) {
				Flags |= CONTACTLIST_ICONFLAG_CHANGE_ICONKIND;
			}
		}


		int GetState() const {
			return CurrentState;
		}

		void SetState(int index);


		const char* GetHint() const {
			return IconHint;
		}

		void SetHint(const char* text) {
			my_cstrdup(IconHint, text);
			if (structSize == 0) {
				Flags |= CONTACTLIST_ICONFLAG_CHANGE_HINT;
			}
		}


		EventHandler* GetEventHandler() {
			return (EventHandler*) HandlerTable::GetInstance()->Get(CallBackFunc);
		}

		void SetEventHandler(EventHandler* handler) {
			CallBackFunc = handler->GetFunction();
			if (structSize == 0) {
				Flags |= CONTACTLIST_ICONFLAG_CHANGE_CALLBACK;
			}
		}


		TLENFUNCTION GetCallback() {
			return CallBackFunc;
		}

		void SetCallback(TLENFUNCTION callback) {
			CallBackFunc = callback;
			if (structSize == 0) {
				Flags |= CONTACTLIST_ICONFLAG_CHANGE_CALLBACK;
			}
		}


		void* GetClientData() {
			return OwnerData;
		}
		
		void SetClientData(void* data) {
			OwnerData = data;
			if (structSize == 0) {
				Flags |= CONTACTLIST_ICONFLAG_CHANGE_OWNERDATA;
			}
		}


		const RECT* Rect() const {
			return &DestRect;
		}


		bool IsMouseOver() {
			return Flags & CONTACTLIST_ICONFLAG_ISMOUSEOVER ? true : false;
		}


		void AddImage(const Image& image);

		void DelImage(int index);

		Image* GetImage(int index);


	friend class ContactItem;
	friend class ContactList;

};


void CopyContactListIconDef(ContactListIconDef* dst, const ContactListIconDef* src);
void FreeContactListIconDef(ContactListIconDef* dst);


} // namespace tlenek


#endif // _TLENEK_CONTACT_ICON_H_
