/* ==========================================================================
 * Name:        tlenek\contactinfo.h
 * Purpose:     Deklaracje klas ContactInfo i ContactInfoList przechowujacych
 *              informacje o danych kontaku
 * Author:      Marcin 'malcom' Malich <me@malcom.pl>
 * Modified by:
 * Created:     24/09/2007 14:30:44
 * SVN-ID:      $Id: contactinfo.h 15 2007-10-16 14:40:16Z malcom $
 * Copyright:   (c) 2007 Marcin 'malcom' Malich <me@malcom.pl>, Poland
 * Licence:     GNU Lesser General Public License
 * ========================================================================== */

#ifndef _TLENEK_CONTACT_INFO_H_
#define _TLENEK_CONTACT_INFO_H_

#include "tlenek\defs.h"
#include "tlenek\utils.h"
#include "tlen_sdk\cl_struct.h"


namespace tlenek {


class TLENEK_DLLIMPEXP ContactInfo : private ContactDef {

	private:

		static bool proc;				// czy pobrano adresy funkcji z tlena

		static ct_init_type ct_init;
		static ct_free_type ct_free;
		static ct_copy_type ct_copy;

		// statyczna metoda, nie inicjuje struct (ct_init), pobiera tylko adresy
		// bo wykorzystywana jest rowniez w 2 funkcjach inline
		static void Init();


	public:

		ContactInfo() {
			Init();
			ct_init(this);
		}

		ContactInfo(const char* contact, const char* protocol) {
			Init();
			ct_init(this);
			SetContact(contact);
			SetProtocol(protocol);
		}

		ContactInfo(const ContactInfo& ct) {
			Init();
			ct_copy(this, (ContactDef*) &ct);
		}

		~ContactInfo() {
			ct_free(this);
		}


		const char* GetContact() const {
			return ContactID;
		}
		
		void SetContact(const char* id) {
			my_cstrdup(ContactID, id);
		}


		const char* GetProtocol() const {
			return ProtocolID;
		}
		
		void SetProtocol(const char* id) {
			my_cstrdup(ProtocolID, id);
		}


		const char* GetFullId() const {
			return FullID;
		}
		
		void SetFullId(const char* id) {
			my_cstrdup(FullID, id);
		}


		const char* GetUser() const {
			return User;
		}
		
		void SetUser(const char* user) {
			my_cstrdup(User, user);
		}


		const char* GetServer() const {
			return Server;
		}
		
		void SetServer(const char* server) {
			my_cstrdup(Server, server);
		}


		const char* GetResource() const {
			return Resource;
		}
		
		void SetResource(const char* resource) {
			my_cstrdup(Resource, resource);
		}


		long GetFlags() const {
			return Flags;
		}

		void SetFlags(long flags) {
			Flags = flags;
		}

		void* GetClientData() const {
			return OwnerData;
		}

		void SetClientData(void* data) {
			OwnerData = data;
		}


	friend class ContactInfoList;
	friend class ContactItem;
	friend class ProtocolMessage;
	friend class ProtocolAuthRequest;
	friend class ProtocolPresence;
	friend class ProtocolContactChanged;

	friend void CopyContactDef(ContactDef* dst, const ContactDef* src);
	friend void FreeContactDef(ContactDef* dst);

};


inline void CopyContactDef(ContactDef* dst, const ContactDef* src) {
	ContactInfo::Init();
	ContactInfo::ct_copy(dst, (ContactDef*) src);
}

inline void FreeContactDef(ContactDef* dst) {
	ContactInfo::Init();
	ContactInfo::ct_free(dst);
}



// cl_add przy kazdym dodaniu reallocuje cala liste
// w tlenie mogliby zrobic to optymlaniej ;p
// dlatego moze zrobimy to po swojemu z malym buforem?
// a moze jednak nie, bo wtedy trzeba bedzie kombinowac kiedy nasz obiekt,
// a kiedy tlena i bufor dzialaby tylko na naszym obiekcie, wiec olac

class TLENEK_DLLIMPEXP ContactInfoList : private ContactsListDef {

	private:

		static bool proc;				// czy pobrano adresy funkcji z tlena

		static cl_init_type cl_init;
		static cl_add_type  cl_add;
		static cl_copy_type cl_copy;
		static cl_free_type cl_free;

		// statyczna metoda, nie inicjuje struct (cl_init), pobiera tylko adresy
		static void Init();


	public:

		ContactInfoList() {
			Init();
			cl_init(this);
		}

		ContactInfoList(const ContactInfoList& cl) {
			Init();
			cl_copy(this, (ContactsListDef*) &cl);
		}

		~ContactInfoList() {
			cl_free(this);
		}


		void Add(const ContactInfo& contact) {
			cl_add(this, (ContactDef*) &contact);
		}

		void Del(int index);

		ContactInfo* Get(int index) const;

		int Find(const char* contact, const char* protocol) const;

		int Size() const {
			return count;
		}

		
		ContactInfo* operator[](int index) const {
			return Get(index);
		}


	friend class ProtocolMessage;

	friend void CopyContactsListDef(ContactsListDef* dst, const ContactsListDef* src);
	friend void FreeContactsListDef(ContactsListDef* dst);

};


inline void CopyContactsListDef(ContactsListDef* dst, const ContactsListDef* src) {
	ContactInfoList::Init();
	ContactInfoList::cl_copy(dst, (ContactsListDef*) src);
}

inline void FreeContactsListDef(ContactsListDef* dst) {
	ContactInfoList::Init();
	ContactInfoList::cl_free(dst);
}


} // namespace tlenek


#endif // _TLENEK_CONTACT_INFO_H_
