/* ==========================================================================
 * Name:        tlenek\contactitem.h
 * Purpose:     Deklaracje bazowej klasy ContactItem reprezentujacej 
 *              pojedynczy element na liscie kontaktow (ContactList)
 *              oraz predefiniowane klasy poszczegolnych elementow
 * Author:      Marcin 'malcom' Malich <me@malcom.pl>
 * Modified by:
 * Created:     20/09/2007 19:57:12
 * SVN-ID:      $Id: contactitem.h 25 2007-11-22 16:43:37Z malcom $
 * Copyright:   (c) 2007 Marcin 'malcom' Malich <me@malcom.pl>, Poland
 * Licence:     GNU Lesser General Public License
 * ========================================================================== */

#ifndef _TLENEK_CONTACT_ITEM_H_
#define _TLENEK_CONTACT_ITEM_H_

#include "tlenek\defs.h"
#include "tlenek\contactinfo.h"
#include "tlenek\contactlist.h"
#include "tlenek\handler.h"
#include "tlenek\tlen.h"
#include "tlen_sdk\plugin_contactlist_defs.h"


namespace tlenek {


class ContactIcon;


enum ContactItemStyle {
// all
	STYLE_UNSELECTABLE		= CONTACTLIST_ITEMFLAG1_UNSELECTABLE,	// czy da si zaznaczac
	STYLE_SELECTASK			= CONTACTLIST_ITEMFLAG1_SELECTCALLBACK,	// pytanie czy moze zaznaczyc element / OnSelect()
	STYLE_OWNERDRAWN		= 0x00002080,	// OWNERDRAWN | OWNERHEIGHT	// malowany w OnDrawn i OnGetHeight
	
//grupa sekcja
	STYLE_NGROUPICON		= CONTACTLIST_ITEMFLAG1_NOGROUPICON,	// czy pokazuje znaczek rozwiniecia
	STYLE_EXPANDABLE		= CONTACTLIST_ITEMFLAG1_EXPANDABLE,		// czy da sie zwinac
	STYLE_AUTOEXPAND		= CONTACTLIST_ITEMFLAG1_AUTOEXPAND,		// auto ro/zwijanie po najechaniu myszka
	STYLE_DBLCLKEXPAND		= CONTACTLIST_ITEMFLAG1_DBLCLKEXPAND,	// ro/zwijanie przy podwojnym kliku
	STYLE_CHILDRENVISIBLE	= CONTACTLIST_ITEMFLAG1_CHILDRENVISIBLE,// widocznosc dzieci mimo niewidocznosci grupy/sekcji
	STYLE_HIDDENIFEMPTY		= CONTACTLIST_ITEMFLAG1_HIDDENIFEMPTY,	// ukryc jesli pusta (bez dzieci) grupa/sekcja

// kontakt
	STYLE_DEFAULTCONTACT	= CONTACTLIST_ITEMFLAG1_DEFAULTCONTACT,	// standardowy kontakt
	STYLE_NDESCLIMIT		= CONTACTLIST_ITEMFLAG1_NODESC_LIMIT,	// nie ucinaj opisu na liscie
	STYLE_HINTNAME			= CONTACTLIST_ITEMFLAG1_HINT_SHOW_NAME,	// pokazuj nazwe kontaktu w hincie
	STYLE_NHINTAVATAR		= CONTACTLIST_ITEMFLAG1_NO_HINT_AVATAR,	// nie pokazuj avatara w hincie

};


TLENEK_DLLIMPEXP extern const long DEFAULT_CONTACTITEMSTYLE;


// rozmiar standardowej jednostki alokacji, dla tablicy ikonek
static const int IconSizeAlloc = 3;


class TLENEK_DLLIMPEXP ContactItem : private ContactListItemDef {

	private:

		int m_icon_alloc;		// ilosc zaalokowanych ikonek w tablicy
		// nie powinnismy odwolywac sie do tej pozycji, jesli rzutujemy
		// z ContactListItemDef na nasz obiekt, bo pole to nie bedzie
		// istniec i dostaniemy AV
		// uzywane jest tylko w czasie tworzenia kontaktu, wiec
		// nie powinny wystapic zadne problemy z tym zwiazane
		

		void Init();


	public:

		enum ContactItemType {
			SECTION,
			GROUP,
			BAR,
			META,
			CONTACT,	
			OTHER
		};

		enum ContactType {
			ONLINE		= CONTACTLIST_ITEMFLAG2_ONLINE,		// dostepny
			OFFLINE		= CONTACTLIST_ITEMFLAG2_OFFLINE,	// niedostepny
			WAITAUTH	= CONTACTLIST_ITEMFLAG2_WAITAUTH,	// oczekiwanie na autoryzacje
			NOTAUTH		= CONTACTLIST_ITEMFLAG2_NOTAUTH,	// brak autoryzacji
			PHONE		= CONTACTLIST_ITEMFLAG2_PHONE,		// numer telefonu
			NOTINLIST	= CONTACTLIST_ITEMFLAG2_NOTINLIST,	// spoza listy kontaktow
			BLOCKED		= CONTACTLIST_ITEMFLAG2_BLOCKED,	// zablokowany
		};


		ContactItem() {
			Init();
		}

		ContactItem(ContactItemType type, int section, int group, const char* name, 
				long style = DEFAULT_CONTACTITEMSTYLE, ContactList::pos pos = ContactList::POS_LAST);

		~ContactItem();


	/* gettery i settery do "manipulacji" dozwolonymi danymi ContactListDef */

	public:

		int GetId() const {
			return ItemHandle;
		}

// id jest przydzielany przez tlen, lepiej go nie zmieniac, bo mozna narobic problemow ;)
//		void SetId(int id) {
//			ItemHandle = id;
//		}


		ContactList::pos GetPosition() const {
			return PreviousItemHandle;
		}

		// ID elementu, po ktorym nalezy wstawic nasz element
		// dodatkowe stale: pos_first i pos_last
		void SetPosition(ContactList::pos pos);


		// set jest chroniony
		ContactItemType GetItemType() const;


		long GetStyle() {
			long draw = 0;
			if (HasFlag(CONTACTLIST_ITEMFLAG1_OWNERDRAWN) || HasFlag(CONTACTLIST_ITEMFLAG1_OWNERHEIGHT)) {
				draw = STYLE_OWNERDRAWN;
			}
			return Flags1 | draw;
		}

		void SetStyle(long style);


		const char* GetMenuId() const {
			return MenuID;
		}

		void SetMenuId(const char* id);


		EventHandler* GetEventHandler() {
			return (EventHandler*) HandlerTable::GetInstance()->Get(CallBackFunc);
		}

		void SetEventHandler(EventHandler* handler);


		TLENFUNCTION GetCallback() {
			return CallBackFunc;
		}

		void SetCallback(TLENFUNCTION callback);


		void* GetClientData() {
			return OwnerData;
		}
		
		void SetClientData(void* data);


	protected:

		// get jest publiczny
		void SetItemType(ContactItemType type);


		bool HasFlag(int flag) const {
			return Flags1 & flag ? true : false;
		}

		void SetFlag(int flag);

		void UnsetFlag(int flag);


		ContactType GetContactType() const {
			return (ContactType) Flags2;
		}

		void SetContactType(const ContactType type);


		int GetSectionId() const {
			return SectionHandle;
		}
		
		void SetSectionId(int id);

		int GetGroupId() const {
			return GroupHandle;
		}
		
		void SetGroupId(int id);

		int GetMetaId() const {
			return MetaGroupHandle;
		}
		
		void SetMetaId(int id);


		const char* GetMainText() const {
			return MainText;
		}
		
		void SetMainText(const char* text);
		
		const char* GetSecondaryText() const {
			return SecondaryText;
		}
		
		void SetSecondaryText(const char* text);
		
		const char* GetLowerText() const {
			return LowerText;
		}
		
		void SetLowerText(const char* text);

		
		const char* GetItemHint() const {
			return ItemHint;
		}
		
		void SetItemHint(const char* text);

		const char* GetItemNewHint() const {
			return ItemNewHint;
		}
		
		void SetItemNewHint(const char* text);


		class ContactInfo* GetContactInfo() {
			return (class ContactInfo*) &ContactInfo;
		}

		void SetContactInfo(const class ContactInfo& data) {
			CopyContactDef(&ContactInfo, &data);
		}


	/* pozostale metody nie operujace bezposrednio na ContactListItemDef */

	public:

		void Select(bool select) {
			if (select) {
				SetFlag(CONTACTLIST_ITEMFLAG1_SELECTED);
			} else {
				UnsetFlag(CONTACTLIST_ITEMFLAG1_SELECTED);
			}
		}

		bool IsSelected() {
			return HasFlag(CONTACTLIST_ITEMFLAG1_SELECTED);
		}

		void Hide(bool hide) {
			if (hide) {
				SetFlag(CONTACTLIST_ITEMFLAG1_HIDDEN);
			} else {
				UnsetFlag(CONTACTLIST_ITEMFLAG1_HIDDEN);
			}
		}

		bool IsHidden() {
			return HasFlag(CONTACTLIST_ITEMFLAG1_HIDDEN);
		}


		void Redraw() {
			if (GetId() == 0) return;
			Tlen::Get()->GetContactList()->RedrawItem(GetId());
		}


	protected:

		void Expand(bool expand) {
			if (expand) {
				SetFlag(CONTACTLIST_ITEMFLAG1_EXPANDED);
			} else {
				UnsetFlag(CONTACTLIST_ITEMFLAG1_EXPANDED);
			}
		}

		bool IsExpanded() {
			return HasFlag(CONTACTLIST_ITEMFLAG1_EXPANDED);
		}


		// operacje na ikonkach //

		void AddIcon(const ContactIcon& icon);

		// usuwa ikonke o podanym id,
		// gdy id zakonczymy "***" to usuwa wszystkie ikonki kontaktu,
		// ktorych id rozpoczyna sie ciagiem poprzedzjacym "***"
		void DelIcon(const char* id);


		void DelIcons() {
			DelIcon("***");
		}


		ContactIcon* GetIcon(const char* id);

		ContactIcon* GetIcon(int index);


  		int GetIconsCount() const {
			return IconsCount;
		}


	friend class ContactList;

};



class TLENEK_DLLIMPEXP Section : public ContactItem {

	public:

		Section() {
			SetItemType(SECTION);
		}

		Section(const char* name, long style = DEFAULT_CONTACTITEMSTYLE, 
				ContactList::pos pos = ContactList::POS_LAST, bool expand = false)
			: ContactItem(SECTION, 0, 0, name, style, pos) {
			Expand(expand);
		}


		void Expand(bool expand) {
			ContactItem::Expand(expand);
		}

		bool IsExpanded() {
			return ContactItem::IsExpanded();
		}


		const char* GetName() const {
			return GetMainText();
		}

		void SetName(const char* name) {
			SetMainText(name);
		}

};


class TLENEK_DLLIMPEXP Group : public ContactItem {

	public:

		Group() {
			SetItemType(GROUP);
		}

		Group(int section, const char* name, long style = DEFAULT_CONTACTITEMSTYLE, 
				ContactList::pos pos = ContactList::POS_LAST, bool expand = false)
			: ContactItem(GROUP, section, 0, name, style, pos) {
			Expand(expand);
		}


		void Expand(bool expand) {
			ContactItem::Expand(expand);
		}

		bool IsExpanded() {
			return ContactItem::IsExpanded();
		}


		int GetSectionId() const {
			return ContactItem::GetSectionId();
		}
		
		void SetSectionId(int id) {
			ContactItem::SetSectionId(id);
		}


		const char* GetName() const {
			return GetMainText();
		}

		void SetName(const char* name) {
			SetMainText(name);
		}


		// opis obok nazwy grupy, np ilosc kontaktow
		const char* GetDescription() const {
			return GetSecondaryText();
		}

		void SetDescription(const char* desc) {
			SetSecondaryText(desc);
		}

};


class TLENEK_DLLIMPEXP Contact : public ContactItem {

	public:

		Contact() {
			SetItemType(CONTACT);
		}

		Contact(int section, int group, const char* name = NULL, long style = 0,//DEFAULT_CONTACTITEMSTYLE, 
				ContactList::pos pos = ContactList::POS_LAST)
			: ContactItem(CONTACT, section, group, name, style, pos) {}


		ContactType GetContactType() const {
			return ContactItem::GetContactType();
		}

		void SetContactType(const ContactType type) {
			ContactItem::SetContactType(type);
		}


		int GetSectionId() const {
			return ContactItem::GetSectionId();
		}
		
		void SetSectionId(int id) {
			ContactItem::SetSectionId(id);
		}

		int GetGroupId() const {
			return ContactItem::GetGroupId();
		}
		
		void SetGroupId(int id) {
			ContactItem::SetGroupId(id);
		}

		int GetMetaId() const {
			return ContactItem::GetMetaId();
		}
		
		void SetMetaId(int id) {
			ContactItem::SetMetaId(id);
		}


		const char* GetName() const {
			return GetMainText();
		}

		void SetName(const char* name) {
			SetMainText(name);
		}

		const char* GetDescription() const {
			return GetLowerText();
		}

		void SetDescription(const char* desc) {
			SetLowerText(desc);
		}


		// tylko dla STYLE_DEFAULT_CONTACT
		const char* GetGroup() const {
			return GetSecondaryText();
		}

		// przypisanie kontaktu do danej grupy,
		// jesli grupa nie istnieje zostanie utworzona
		// tylko dla STYLE_DEFAULT_CONTACT
		void SetGroup(const char* group) {
			SetSecondaryText(group);
		}


		const char* GetHint() const {
			return ContactItem::GetItemHint();
		}		
		
		void SetHint(const char* text) {
			ContactItem::SetItemHint(text);
		}

		const char* GetNewHint() const {
			return ContactItem::GetItemNewHint();;
		}
		
		void SetNewHint(const char* text) {
			ContactItem::SetItemNewHint(text);
		}


		class ContactInfo* GetContactInfo() {
			return ContactItem::GetContactInfo();
		}

		void SetContactInfo(const class ContactInfo& data) {
			ContactItem::SetContactInfo(data);
		}


	// operacje na ikonkach //

		void AddIcon(const ContactIcon& icon) {
			ContactItem::AddIcon(icon);
		}

		void DelIcon(const char* id) {
			ContactItem::DelIcon(id);
		}

		void DelIcons() {
			ContactItem::DelIcons();
		}

		ContactIcon* GetIcon(int index) {
			return ContactItem::GetIcon(index);
		}

		int GetIconsCount() const {
			return ContactItem::GetIconsCount();
		}

};


class TLENEK_DLLIMPEXP Meta : private ContactItem {

};


class TLENEK_DLLIMPEXP Other : private ContactItem {

};


} // namespace tlenek


#endif // _TLENEK_CONTACT_ITEM_H_
