/* ==========================================================================
 * Name:        tlenek\contactlist.h
 * Purpose:     Deklaracja klasy ContactList odpowiedzialnej za obsluge 
 *              listy kontaktow Tlena
 * Author:      Marcin 'malcom' Malich <me@malcom.pl>
 * Modified by:
 * Created:     20/09/2007 18:24:48
 * SVN-ID:      $Id: contactlist.h 15 2007-10-16 14:40:16Z malcom $
 * Copyright:   (c) 2007 Marcin 'malcom' Malich <me@malcom.pl>, Poland
 * Licence:     GNU Lesser General Public License
 * ========================================================================== */

#ifndef _TLENEK_CONTACT_LIST_H_
#define _TLENEK_CONTACT_LIST_H_

#include "tlenek\defs.h"
#include "tlenek\tlen.h"
#include "tlenek\function.h"
#include "tlenek\stringlist.h"
#include "tlen_sdk\plugin_contactlist_defs.h"


namespace tlenek {


class ContactItem;
class ContactIcon;
class ContactInfo;


class TLENEK_DLLIMPEXP ContactList {

	private:

		TLENFUNCTION sms_number;		// TPA_GET_SMS_NUMBER
		TLENFUNCTION sms_contact;		// TPA_GET_SMS_NUMBER_CONTACT
		TLENFUNCTION contact_name;		// TPA_GET_CONTACT_NAME
		TLENFUNCTION contact_desc;		// TPA_GET_CONTACT_STATUS_TEXT

		StringList m_sections;
		StringList m_groups;

		ContactList();
		~ContactList() {}


	public:

		typedef int pos;
		static const pos POS_FIRST;
		static const pos POS_LAST;


		// dodaje element do listy kontaktow
		// zwraca id dodanego elementu
		int AddItem(const ContactItem& item);

		// usuwa element z listy kontaktow
		void DelItem(int id);

		// szuka elementu na liscie kontaktow i zwraca jego id
		int FindItem(const ContactInfo& contact) {
			return Tlen::Get()->GetFunction()->CallTS(TLEN_CONTACTLIST_FINDITEM, (WPARAM) &contact, NULL);
		}

		// zwraca element z listy kontaktow o podanym id
		ContactItem* GetItem(int id) {
			return (ContactItem*) Tlen::Get()->GetFunction()->CallTS(TLEN_CONTACTLIST_GETITEM, (WPARAM) id, NULL);
		}

		
		enum UpdateItemFlag {
			NONE = 0,
			REDRAWITEM = CONTACTLIST_ITEMCHANGEFLAG_REDRAWITEM,		// przemalowac pozycje po update?
			NOLISTUPDATE = CONTACTLIST_ITEMCHANGEFLAG_NOLISTUPDATE,	// nie odswiezac (redraw) listy kontaktow
		};

		// updatuje element o podanym id, dane do zmiany w obiekcie
		void UpdateItem(int id, const ContactItem& item, UpdateItemFlag flag = REDRAWITEM);


		// odrysowanie elementu o podanym id
		void RedrawItem(int id) {
			Tlen::Get()->GetFunction()->CallTS(TLEN_CONTACTLIST_REDRAWITEM, (WPARAM) id, NULL);
		}

		// odrysowanie listy kontaktow
		void Redraw() {
			Tlen::Get()->GetFunction()->CallTS(TLEN_CONTACTLIST_REDRAWLIST, NULL, NULL);
		}


		enum PropetyType_t {
			StatusVisible = 1,
			GroupVisible,
			OfflineVisible,
			DescVisible,
			DrawEnable	// nie dziala w tlenie
		};

		// zwraca wartosc danego parametru
		int GetProperty(PropetyType_t property) {
			int value;
			Tlen::Get()->GetFunction()->CallTS(TLEN_CONTACTLIST_GETPROPERTY, (WPARAM) property, (LPARAM) &value);
			return value;
		}

		// ustawia wartosc danego parametru
		void SetProperty(PropetyType_t property, int value) {
			Tlen::Get()->GetFunction()->CallTS(TLEN_CONTACTLIST_GETPROPERTY, (WPARAM) property, (LPARAM) &value);
		}


		// ikonki

		// dodaje ikonke do elementu o podanym id
		void AddIcon(int id, const ContactIcon& icon);

		// usuwa ikonke o podanym icon_id z elementu o podanym id
		// gdy icon_id zakonczymy "***" to usuwa wszystkie ikonki kontaktu,
		// ktorych id rozpoczyna sie podanym ciagiem znakow
		void DelIcon(int id, const char* icon_id);

		// usuwa ikonki z elementu o podanym id,
		// usuwane sa ikonki o id rozpoczynajacym sie od ciagu znakow okreslonym w mask,
		// gdy NULL to usunie wszystkie
		void DelIcons(int id) {
			DelIcon(id, "***");
		}

		void UpdateIcon(int id, const ContactIcon& icon);

	// inne //

		// zwraca numer smsowy podanego kontaktu
		// lub NULL gdy kontakt nie posiada przypisanego numeru
		const char* GetSmsNumber(const ContactInfo& ci) {
			return (char*) sms_number((WPARAM) &ci, NULL);
		}

		// zwraca wskaznik na ContactInfo kontaktu o podanym numerze smsowym 
		// lub NULL w przypadku, gdy nie znaleziono takiego numeru
		const ContactInfo* GetSmsContact(const char* number) {
			return (ContactInfo*) sms_contact((WPARAM) number, NULL);
		}

		// zwraca nazwe danego kontaktu
		const char* GetContactName(const ContactInfo& ci) {
			return (char*) contact_name((WPARAM) &ci, NULL);
		}

		// zwraca opis danego kontaktu
		const char* GetContactDesc(const ContactInfo& ci) {
			return (char*) contact_desc((WPARAM) &ci, NULL);
		}


		// zwraca liste nazw sekcji
		const StringList* GetSections();
		
		// zwraca liste nazw grup dla podanej sekcji
		// jesli nie podano sekcji, zwraca dla "Lista kontaktow"
		const StringList* GetGroups(int section = 1);


		ContactItem* operator[](int id) {
			return GetItem(id);
		}


	private:

		static int EnumCallbackSections(WPARAM wParam, LPARAM lParam);
		static int EnumCallbackGroups(WPARAM wParam, LPARAM lParam);


	friend class Tlen;

};


} // namespace tlenek


#endif // _TLENEK_CONTACT_LIST_H_
