/* ==========================================================================
 * Name:        tlenek\event.h
 * Purpose:     Deklaracje klas i szablonow niezbednych do obslugi zdarzen
 * Author:      Marcin 'malcom' Malich <me@malcom.pl>
 * Modified by:
 * Created:     21/09/2007 23:42:25
 * SVN-ID:      $Id: event.h 18 2007-10-19 15:08:56Z malcom $
 * Copyright:   (c) 2007 Marcin 'malcom' Malich <me@malcom.pl>, Poland
 * Licence:     GNU Lesser General Public License
 * ========================================================================== */

#ifndef _TLENEK_EVENT_H_
#define _TLENEK_EVENT_H_

#include "tlenek\defs.h"
#include "tlenek\handler.h"
#include "tlen_sdk\plugin_contactlist_defs.h"
#include "tlen_sdk\plugin_protocol_defs.h"


namespace tlenek {


// templates for event handlers //

class ContactIcon;
class ContactItem;
class OptionPage;


class TLENEK_DLLIMPEXP ContactIconEvt : public EventHandler {

	public:

		int Process();

		ContactItem* GetContactItem() {
			return (ContactItem*) GetEventObject();
		}

		ContactIcon* GetContactIcon() {
			return (ContactIcon*) ((ItemCallBackMouseDef*) GetEventData())->Icon;
		}


		// dla myszki
		virtual void OnMouseExit(POINT position, int button) {}
		virtual void OnMouseMove(POINT position, int button) {}
		virtual void OnMouseDown(POINT position, int button) {}
		virtual void OnMouseUp(POINT position, int button) {}
		virtual void OnMouseDbClick(POINT position, int button) {}

};


template<class T>
class ContactIconEventHandler : public ContactIconEvt {

	public:

		ContactIconEventHandler() {
			AddToTable();
		}

		~ContactIconEventHandler() {
			DelFromTable();
		}


		TLENFUNCTION GetFunction() {
			return Callback;
		}


	private:

		static int Callback(WPARAM wParam, LPARAM lParam) {
			TLENEK_AFX
			EventHandler* obj = (EventHandler*) HandlerTable::GetInstance()->Get(Callback);
			if (obj == NULL) return 0;

			ItemCallBackEventDef* def = (ItemCallBackEventDef*) wParam;

			obj->SetEventType(def->Event);
			obj->SetEventData((void*) lParam);
			obj->SetEventObject(def->Item);

			return obj->Process();
		}

};


class TLENEK_DLLIMPEXP ContactEvt : public EventHandler {

	public:

		int Process();

		ContactItem* GetContactItem() {
			return (ContactItem*) GetEventObject();
		}


		// dla OwnerDrawn
		virtual int OnGetHeight() { return 0; }
		virtual void OnDraw(HDC hdc, RECT* rect) {}

		// czy mozna zaznaczyc dany zestaw?
		virtual int OnSelect() { return 0; }

		// dla myszki
		virtual void OnMouseExit(POINT position, int button) {}
		virtual void OnMouseMove(POINT position, int button) {}
		virtual void OnMouseDown(POINT position, int button) {}
		virtual void OnMouseUp(POINT position, int button) {}
		virtual void OnMouseDbClick(POINT position, int button) {}

		// dla klawiatury
		virtual void OnKeyDown(int key, int data) {}
		virtual void OnKeyUp(int key, int data) {}

};


template<class T>
class ContactEventHandler : public ContactEvt {

	public:

		ContactEventHandler() {
			AddToTable();
		}

		~ContactEventHandler() {
			DelFromTable();
		}


		TLENFUNCTION GetFunction() {
			return Callback;
		}


	private:

		static int Callback(WPARAM wParam, LPARAM lParam) {
			TLENEK_AFX
			EventHandler* obj = (EventHandler*) HandlerTable::GetInstance()->Get(Callback);
			if (obj == NULL) return 0;

			ItemCallBackEventDef* def = (ItemCallBackEventDef*) wParam;

			obj->SetEventType(def->Event);
			obj->SetEventData((void*) lParam);
			obj->SetEventObject(def->Item);

			return obj->Process();
		}

};


class TLENEK_DLLIMPEXP OptionPageEvt : public EventHandler {

	public:

		int Process();

		OptionPage* GetOptionPage() {
			return (OptionPage*) GetEventObject();
		}


		// akcje
		virtual HWND OnCreate(HWND parent) { return 0; }
		virtual void OnDestroy(HWND hwnd) {}

		virtual void OnActionShow() {}
		virtual void OnActionHide() {}
		virtual void OnClickOk() {}
		virtual void OnClickApply() {}
		virtual void OnClickCancel() {}

};


template<class T>
class OptionPageEventHandler : public OptionPageEvt {

	public:

		OptionPageEventHandler() {
			AddToTable();
		}

		~OptionPageEventHandler() {
			DelFromTable();
		}


		TLENFUNCTION GetFunction() {
			return Callback;
		}


	private:

		static int Callback(WPARAM wParam, LPARAM lParam) {
			TLENEK_AFX
			EventHandler* obj = (EventHandler*) HandlerTable::GetInstance()->Get(Callback);
			if (obj == NULL) return 0;

			TlenOptionPageShowInfo* info = (TlenOptionPageShowInfo*) wParam;

			obj->SetEventType(info->Action);
			obj->SetEventData((void*) info);
			obj->SetEventObject(info->Page->OwnerData);

			return obj->Process();
		}

};



// templates for hook event handlers //

class ContactInfo;
class ProtocolMessage;
class ProtocolPresence;
class ProtocolAuthRequest;


template<class T>
class AppMessageEventHandler : public EventHandler {

	public:

		AppMessageEventHandler() {
			AddToTable();
		}

		~AppMessageEventHandler() {
			DelFromTable();
		}


		TLENFUNCTION GetFunction() {
			return Callback;
		}

		int Process() {
			MSG* msg = (MSG*) GetEventData();
			return OnMessage(msg->hwnd, msg->message, msg->wParam, msg->lParam);
			return 0;
		};


		// akcje
		virtual int OnMessage(HWND hwnd, UINT mesg, WPARAM wParam, LPARAM lParam) { return 0; }


	private:

		static int Callback(WPARAM wParam, LPARAM lParam) {
			TLENEK_AFX
			EventHandler* obj = (EventHandler*) HandlerTable::GetInstance()->Get(Callback);
			if (obj == NULL) return 0;

			MSG* msg = (MSG*) wParam;

			obj->SetEventType(msg->message);
			obj->SetEventData((void*) wParam);
			obj->SetEventObject(msg->hwnd);

			return obj->Process();
		}

};


template<class T>
class PluginsLoadedEventHandler : public EventHandler {

	public:

		PluginsLoadedEventHandler() {
			AddToTable();
		}

		~PluginsLoadedEventHandler() {
			DelFromTable();
		}


		TLENFUNCTION GetFunction() {
			return Callback;
		}

		int Process() {
			OnLoaded();
			return 0;
		}


		// akcje
		virtual void OnLoaded() {}


	private:

		static int Callback(WPARAM wParam, LPARAM lParam) {
			TLENEK_AFX
			EventHandler* obj = (EventHandler*) HandlerTable::GetInstance()->Get(Callback);
			if (obj == NULL) return 0;

			return obj->Process();
		}

};


class TLENEK_DLLIMPEXP ContactListEvt : public EventHandler {

	public:

		int Process();


		// akcje
		virtual int OnBeforeAdd(const ContactItem* item) { return 0; }
		virtual int OnAfterAdd(const ContactItem* item) { return 0; }
		virtual int OnBeforeDel(const ContactItem* item) { return 0; }
		virtual int OnAfterDel(const ContactItem* item) { return 0; }

		virtual int OnBeforeChange(const ContactItem* old_item, const ContactItem* new_item) { return 0; }
		virtual int OnAfterChange(const ContactItem* old_item, const ContactItem* new_item) { return 0; }

};


template<class T>
class ContactListEventHandler : public ContactListEvt {

	public:

		ContactListEventHandler() {
			AddToTable();
		}

		~ContactListEventHandler() {
			DelFromTable();
		}


		TLENFUNCTION GetFunction() {
			return Callback;
		}


	private:

		static int Callback(WPARAM wParam, LPARAM lParam) {
			TLENEK_AFX
			EventHandler* obj = (EventHandler*) HandlerTable::GetInstance()->Get(Callback);
			if (obj == NULL) return 0;

			obj->SetEventType(wParam);
			obj->SetEventData((void*) lParam);

			return obj->Process();
		}

};


class TLENEK_DLLIMPEXP ProtocolEvt : public EventHandler {

	public:

		int Process();

		const char* GetProtocol() {
			return (const char*) GetEventObject();
		}


		// akcje
		virtual int OnBeforeRecvRawData(char* data, unsigned int len) { return 0; }
		virtual void OnAfterRecvRawData(char* data, unsigned int len) {}
		virtual void OnAfterProcessedRecvRawData(char* data, unsigned int len) {}
		virtual void OnAfterRecvMessage(char* data, unsigned int len) {}
		virtual int OnBeforeSendRawData(char* data, unsigned int len) { return 0; }
		virtual void OnAfterSendRawData(char* data, unsigned int len) {}

		virtual void OnLoggedIn(const ContactInfo* ct) {}
		virtual void OnLoggedOut(const ContactInfo* ct) {}

		virtual int OnRecvMessage(const ProtocolMessage* msg) { return 0; }
		virtual int OnBeforeSendMessage(const ProtocolMessage* msg) { return 0; }

		virtual int OnRecvPresence(const ProtocolPresence* prs) { return 0; }
		virtual int OnBeforeSendPresence(const ProtocolPresence* prs) { return 0; }

		virtual void OnAuthRequest(const ProtocolAuthRequest* auth) {}
		virtual void OnRecvRoster() {}

		virtual int OnContactDeleted(const ContactInfo* ct) { return 0; }
		virtual int OnCcontactChanged(ContactInfo* old_ci, ContactInfo* new_ci) { return 0; }

};


template<class T>
class ProtocolEventHandler : public ProtocolEvt {

	public:

		ProtocolEventHandler() {
			AddToTable();
		}

		~ProtocolEventHandler() {
			DelFromTable();
		}


		TLENFUNCTION GetFunction() {
			return Callback;
		}


	private:

		static int Callback(WPARAM wParam, LPARAM lParam) {
			TLENEK_AFX
			EventHandler* obj = (EventHandler*) HandlerTable::GetInstance()->Get(Callback);
			if (obj == NULL) return 0;

			ProtocolEventDef* event = (ProtocolEventDef*) wParam;

			obj->SetEventType(event->EventID);
			obj->SetEventData((void*) lParam);
			obj->SetEventObject(event->ProtocolID);

			return obj->Process();
		}

};


} // namespace tlenek


#endif // _TLENEK_EVENT_H_
