/* ==========================================================================
 * Name:        tlenek\function.h
 * Purpose:     Deklaracja klasy Function zarzadzajacej funkcjami tlenowymi
 * Author:      Marcin 'malcom' Malich <me@malcom.pl>
 * Modified by:
 * Created:     04/10/2007 19:06:31
 * SVN-ID:      $Id: function.h 16 2007-10-17 20:40:54Z malcom $
 * Copyright:   (c) 2007 Marcin 'malcom' Malich <me@malcom.pl>, Poland
 * Licence:     GNU Lesser General Public License
 * ========================================================================== */

#ifndef _TLENEK_FUNCTION_H_
#define _TLENEK_FUNCTION_H_

#include "tlenek\defs.h"
#include "tlenek\misc.h"
#include "tlenek\handler.h"
#include "tlenek\utils.h"
#include "tlen_sdk\plugin_procaddr.h"
#include <list>


namespace tlenek {


class TLENEK_DLLIMPEXP Function {

	private:

		typedef std::list<HANDLE> en_list_t;
		en_list_t m_list;

		Function() {}
		~Function() {}


	public:

		int Create(const char* name, TLENFUNCTION function);

		int Create(const char* name, FunctionHandler* handler) {
			return Create(name, handler->GetFunction());
		}

		void Destroy(int id);

		void DestroyAll();


		int Call(const char* name, WPARAM wParam, LPARAM lParam) {
			return GetTlenFunctions()->CallTlenFunction(GetPluginInstance(), name, wParam, lParam);
		}

		int CallTS(const char* name, WPARAM wParam, LPARAM lParam) {
			return GetTlenFunctions()->CallTlenFunctionTS(GetPluginInstance(), name, wParam, lParam);
		}

		int CallProto(const char* proto, const char* name, WPARAM wParam, LPARAM lParam);


		bool Exists(const char* name) {
			return GetTlenFunctions()->TlenFunctionExists(GetPluginInstance(), name) == 1;
		}


		int Count() {
			return m_list.size();
		}


		TLENFUNCTION GetProcAddr(int proc) {
			return (TLENFUNCTION) GetTlenFunctions()->CallTlenFunctionTS(GetPluginInstance(), TLEN_GET_PROC_ADDRESS, proc, NULL);
		}


	friend class Tlen;

};


// templates for function handlers //

template<class T>
class DefaultFunctionHandler : public FunctionHandler {

	public:

		DefaultFunctionHandler() {
			AddToTable();
		}

		~DefaultFunctionHandler() {
			DelFromTable();
		}


		TLENFUNCTION GetFunction() {
			return Callback;
		}

		int Process() {
			return OnFunction(GetWparam(), GetLparam());
		}


		// akcje
		virtual int OnFunction(WPARAM wParam, LPARAM lParam) { return 0; }


	private:

		static int Callback(WPARAM wParam, LPARAM lParam) {
			FunctionHandler* obj = (FunctionHandler*) HandlerTable::GetInstance()->Get(Callback);
			if (obj == NULL) return 0;

			obj->SetWparam(wParam);
			obj->SetLparam(lParam);

			return obj->Process();
		}

};


} // namespace tlenek


#endif // _TLENEK_FUNCTION_H_
