/* ==========================================================================
 * Name:        tlenek\handler.h
 * Purpose:     Deklaracja klasy bazowej EventHandler dla zdarzen (event)
 *              EventHandlerTable zawierajacej instancje uzywanych handlerow
 * Author:      Marcin 'malcom' Malich <me@malcom.pl>
 * Modified by:
 * Created:     28/09/2007 13:49:48 (extracted from tlenek\event.h)
 * SVN-ID:      $Id: handler.h 25 2007-11-22 16:43:37Z malcom $
 * Copyright:   (c) 2007 Marcin 'malcom' Malich <me@malcom.pl>, Poland
 * Licence:     GNU Lesser General Public License
 * ========================================================================== */

#ifndef _TLENEK_HANDLER_H_
#define _TLENEK_HANDLER_H_

#include "tlenek\defs.h"
#include <list>
#include <windows.h>


namespace tlenek {


class Handler;


class TLENEK_DLLIMPEXP HandlerTable {

	private:

		static HandlerTable* instance;

		struct en_t {
			TLENFUNCTION function;
			Handler* instance;

			int operator==(const en_t& entry) {
				return entry.function == function;
			}
		};

		typedef std::list<en_t> en_list_t;
		en_list_t m_list;

		HandlerTable() {}
		~HandlerTable() {}


	public:

		static HandlerTable* GetInstance();

		static void FreeInstance() {
			delete instance;
			instance = NULL;
		}


		void Add(TLENFUNCTION function, Handler* instance);

		void Del(TLENFUNCTION function);

		Handler* Get(TLENFUNCTION function);

		int Count() {
			return m_list.size();
		}

};


class TLENEK_DLLIMPEXP Handler {

	public:

		virtual TLENFUNCTION GetFunction() {
			return 0;
		}

		virtual int Process() {
			MessageBoxA(NULL, "Process", "Handler", MB_OK);
			return 0;
		}


	protected:

		void AddToTable() {
			HandlerTable::GetInstance()->Add(GetFunction(), this);
		}

		void DelFromTable() {
			HandlerTable::GetInstance()->Del(GetFunction());
		}

};


class TLENEK_DLLIMPEXP EventHandler : public Handler {

	private:

		int m_type;
		void* m_data;
		void* m_object;


	public:

		EventHandler() {
			m_type = 0;
			m_data = NULL;
			m_object = NULL;
		}


		int GetEventType() const {
			return m_type;
		}

		void SetEventType(int type) {
			m_type = type;
		}


		void* GetEventData() const {
			return m_data;
		}

		void SetEventData(void* data) {
			m_data = data;
		}


		void* GetEventObject() const {
			return m_object;
		}

		void SetEventObject(void* object) {
			m_object = object;
		}

};


class TLENEK_DLLIMPEXP FunctionHandler : public Handler {

	private:

		WPARAM m_wParam;
		LPARAM m_lParam;


	public:

		FunctionHandler() {
			m_wParam = 0;
			m_lParam = 0;
		}


		WPARAM GetWparam() const {
			return m_wParam;
		}

		void SetWparam(WPARAM wParam) {
			m_wParam = wParam;
		}


		LPARAM GetLparam() const {
			return m_lParam;
		}

		void SetLparam(LPARAM lParam) {
			m_lParam = lParam;
		}

};


} // namespace tlenek


#endif // _TLENEK_HANDLER_H_
