/* ==========================================================================
 * Name:        tlenek\hook.h
 * Purpose:     Deklaracja klasy Hook zarzadzajacej obsluga hookow
 * Author:      Marcin 'malcom' Malich <me@malcom.pl>
 * Modified by:
 * Created:     28/09/2007 13:34:04
 * SVN-ID:      $Id: hook.h 16 2007-10-17 20:40:54Z malcom $
 * Copyright:   (c) 2007 Marcin 'malcom' Malich <me@malcom.pl>, Poland
 * Licence:     GNU Lesser General Public License
 * ========================================================================== */

#ifndef _TLENEK_HOOK_H_
#define _TLENEK_HOOK_H_

#include "tlenek\defs.h"
#include "tlenek\misc.h"
#include "tlenek\handler.h"
#include "tlenek\utils.h"
#include <list>


namespace tlenek {


class TLENEK_DLLIMPEXP Hook {

	public:

		enum HookType {
			ALL,					// wykorzystywane tylko przy unhooku
			APP_MESSAGE,
			PLUGINS_LOADED,
			CONTACTLIST,
			ALL_PROTOCOL,
			PROTOCOL,
			MENU,					// nie zaimplementowane
			SKIN,					// nie zaimplementowane
			SOUND,					// nie zaimplementowane
		};


	private:

		struct en_t {
			HookType type;
			HANDLE handle;		// tlenowe handle hooka
			char* param;		// parametry dodatkowe, id protokolu, jesli hook go dotyczy etc

			en_t() {}

			en_t(const en_t& entry) {
				type = entry.type;
				handle = entry.handle;
				my_strdup(param, entry.param, false);
			}
			
			~en_t() {
				delete [] param;
			}

			int operator==(const en_t& entry) {
				return (type == entry.type && strcmp(param, entry.param) == 0 && handle == entry.handle);
			}

		};

		typedef std::list<en_t> en_list_t;
		en_list_t m_list;

		typedef std::list<HANDLE> h_list_t;
		h_list_t m_h_list;

		Hook() {}
		~Hook() {
			// zwolnic wszystkie hooki?
			// NIE! niech sie wysypie tlen jak user zapomni sam zwolnic :P
		}


	public:

		// UWAGA: zwracany id hooka nie jest uchwytem jaki zwraca tlenowe api przy hookowaniu
		int Attach(HookType type, TLENFUNCTION function, const char* param = NULL);

		int Attach(HookType type, EventHandler* handler, const char* param = NULL) {
			return Attach(type, handler->GetFunction(), param);
		}

		void Detach(int id);

		void DetachAll(HookType type = ALL);


		int Create(const char* name);

		void Destroy(int id);

		void DestroyAll();


		void Notify(int id, WPARAM wParam, LPARAM lParam) {
			GetTlenFunctions()->NotifyTlenHooks(GetPluginInstance(), (HANDLE) id, wParam, lParam);
		}

		void NotifyProto(const char* proto, enum ProtocolEventType event_type, void* even_data);


		int CountAttached() {
			return m_list.size();
		}

		int CountCreated() {
			return m_h_list.size();
		}


	private:

		void detach(en_list_t::iterator& it);


	friend class Tlen;

};


} // namespace tlenek


#endif // _TLENEK_HOOK_H_
