/* ==========================================================================
 * Name:        tlenek\info.h
 * Purpose:     Deklaracje funkcji zawartych w przestrzeni nazw 'info' 
 *              dostarczajacych roznych przydatnych informacji
 * Author:      Marcin 'malcom' Malich <me@malcom.pl>
 * Modified by:
 * Created:     30/09/2007 02:52:45 (extracted from tlenek\misc.h)
 * SVN-ID:      $Id: info.h 5 2007-10-05 14:03:42Z malcom $
 * Copyright:   (c) 2007 Marcin 'malcom' Malich <me@malcom.pl>, Poland
 * Licence:     GNU Lesser General Public License
 * ========================================================================== */

#ifndef _TLENEK_INFO_H_
#define _TLENEK_INFO_H_

#include <stdio.h>
#include "tlenek\defs.h"
#include "tlenek\tlen.h"
#include "tlenek\function.h"
#include "tlenek\version.h"
#include "tlenek\plugin.h"
#include "tlen_sdk\plugin_procaddr.h"


namespace tlenek {
	namespace info {


	inline DWORD GetTlenekVersion() {
		return TLENEK_VERSION;
	}

	inline const char* GetTlenekVersionStr() {
		return TLENEK_VERSION_STRING;
	}

	// mozna tez skorzystac z TLEN_GET_TLEN_VERSION
	inline DWORD GetTlenVersion() {
		return Plugin::GetInstance()->m_tlen_ver;
	}

	static char tlen_version_str[24];

	inline const char* GetTlenVersionStr() {
		DWORD ver = GetTlenVersion();
		sprintf(tlen_version_str, "Tlen.pl %d.%d.%d.%d", ver>>24&0xFF, ver>>16&0xFF, ver>>8&0xFF, ver&0xFF);
		return tlen_version_str;
	}

	inline DWORD GetApiVersion() {
		return PLUGIN_API_VERSION;
	}

	inline DWORD GetPluginVersion() {
		return Plugin::GetInstance()->GetPluginVersion();
	}

	inline int GetProtocolVersion() {
		return Tlen::Get()->GetFunction()->CallTS(TLEN_GET_TLEN_PROTOCOL_VERSION, NULL, NULL);
	}


	static char* tlen_path = NULL;

	// zwraca katalog instalacyjny tlenu
	// zaalokowany bufor zostaje automatycznie zwolniony w destruktorze pluginu
	TLENEK_DLLIMPEXP const char* GetTlenPath();

	// prywtana (moze niekoniecznie?) funkcja zwalniajaca bufor z sciezka do tlena
	inline void FreeTlenPath() {
		delete [] tlen_path;
	}


	typedef TLENPLUGININFO PluginInfo;

	inline PluginInfo* GetPluginInfo() {
		return Plugin::GetInstance()->GetPluginInfo();
	}

	inline const char* GetPluginName() {
		return Plugin::GetInstance()->GetPluginName();
	}

	inline const char* GetPluginAuthor() {
		return Plugin::GetInstance()->GetPluginAuthor();
	}


	typedef ProfInfoDef ProfileInfo;
	typedef ProfileInfoDef ProfileInfoEx;

	TLENEK_DLLIMPEXP ProfileInfo* GetProfileInfo();
	TLENEK_DLLIMPEXP ProfileInfoEx* GetProfileInfoEx();

	inline const char* GetLogin() {
		const ProfileInfo* prof = GetProfileInfo();
		if (prof == NULL) return NULL;
		return prof->Login;
	}

	inline const char* GetProfileName() {
		const ProfileInfo* prof = GetProfileInfo();
		if (prof == NULL) return NULL;
		return prof->ProfileName;
	}

	inline const char* GetProfilePath() {
		const ProfileInfo* prof = GetProfileInfo();
		if (prof == NULL) return NULL;
		return prof->ProfileDirPath;
	}


	} // namespace info
} // namespace tlenek


#endif // _TLENEK_INFO_H_
