/* ==========================================================================
 * Name:        tlenek\inifile.h
 * Purpose:     Deklaracje klasy IniFile ulatwiajacej operacje na plikach ini
 * Author:      Marcin 'malcom' Malich <me@malcom.pl>
 * Modified by:
 * Created:     18/10/2007 10:52:45
 * SVN-ID:      $Id: inifile.h 17 2007-10-18 13:01:32Z malcom $
 * Copyright:   (c) 2007 Marcin 'malcom' Malich <me@malcom.pl>, Poland
 * Licence:     GNU Lesser General Public License
 * ========================================================================== */

#ifndef _TLENEK_INI_FILE_H_
#define _TLENEK_INI_FILE_H_

#include <stdio.h>
#include <windows.h>
#include "tlenek\defs.h"
#include "tlenek\stringlist.h"
#include "tlenek\utils.h"


namespace tlenek {


static const int SizeAlloc = 256;


class TLENEK_DLLIMPEXP IniFile {

	private:

		char* m_file;
		char* m_buf;
		size_t m_size;
		StringList m_sl;


	public:

		IniFile() {
			m_file = NULL;
			m_buf = new char[SizeAlloc];
			m_size = SizeAlloc;
			Load();
		}

		IniFile(const char* file) {
			m_file = NULL;
			m_buf = new char[SizeAlloc];
			m_size = SizeAlloc;
			Load(file);
		}

		IniFile(const IniFile& ini) {
			my_strdup(m_file, ini.m_file, false);
			m_buf = new char[ini.m_size];
			m_size = ini.m_size;
		}


		~IniFile() {
			delete [] m_file;
			delete [] m_buf;
		}


		void Load(const char* file = NULL);


		const char* GetString(const char* section, const char* key, const char* def = NULL);

		bool SetString(const char* section, const char* key, const char* value) {
			return WritePrivateProfileString(section, key, value, m_file) != 0;
		}


		int GetInt(const char* section, const char* key, int def = 0) {
			return GetPrivateProfileInt(section, key, def, m_file);
		}

		bool SetInt(const char* section, const char* key, int value) {
			sprintf(m_buf, "%d", value);
			return SetString(section, key, m_buf);
		}


		bool GetBool(const char* section, const char* key, bool def = false) {
			return GetInt(section, key, def) != 0;
		}

		bool SetBool(const char* section, const char* key, bool value) {
			return SetInt(section, key, value);
		}


		bool GetStruct(const char* section, const char* key, void* data, int size) {
			return GetPrivateProfileStruct(section, key, data, size, m_file) != 0;
		}

		bool SetStruct(const char* section, const char* key, void* data, int size) {
			return WritePrivateProfileStruct(section, key, data, size, m_file) != 0;
		}


		StringList* GetSection(const char* section);

		bool SetSection(const char* section, const StringList& sl);


		StringList* GetSectionNames();
		
		StringList* GetKeyNames(const char* section);


		bool DelSection(const char* section) {
			return SetString(section, NULL, NULL);
		}

		bool DelKey(const char* section, const char* key) {
			return SetString(section, key, NULL);
		}


		private:

			void realloc_buf() {
				delete [] m_buf;
				m_size += SizeAlloc;
				m_buf = new char[m_size];
			}

};


} // namespace tlenek


#endif // _TLENEK_INI_FILE_H_
