/* ==========================================================================
 * Name:        tlenek\optionswindow.h
 * Purpose:     Deklaracje klas zwiazanych z reprezentacja i obsluga okienek 
 *              dialogowych dla ustawien pluginu
 * Author:      Marcin 'malcom' Malich <me@malcom.pl>
 * Modified by:
 * Created:     28/09/2007 23:40:57
 * SVN-ID:      $Id: optionswindow.h 20 2007-11-08 15:38:29Z malcom $
 * Copyright:   (c) 2007 Marcin 'malcom' Malich <me@malcom.pl>, Poland
 * Licence:     GNU Lesser General Public License
 * ========================================================================== */

#ifndef _TLENEK_OPTIONS_WINDOW_H_
#define _TLENEK_OPTIONS_WINDOW_H_

#include <windows.h>
#include "tlenek\defs.h"
#include "tlenek\image.h"
#include "tlenek\misc.h"
#include "tlenek\tlen.h"
#include "tlenek\function.h"
#include "tlenek\handler.h"
#include "tlenek\utils.h"
#include "tlen_sdk\plugin_options.h"


namespace tlenek {


enum OptionPageStyle {
	STYLE_TLENCOLOR = TLEN_OPTIONS_PAGEDEF_FLAG_TLENCOLOR,	// okienko w kolorze tlenowym
	STYLE_ICONTRANS = TLEN_OPTIONS_PAGEDEF_FLAG_ICONTRANS,	// ikonka z przezroczystym tlem
	STYLE_BASIC = TLEN_OPTIONS_PAGEDEF_FLAG_BASICOPT,		// pozycja dla trybu podstawowego opcji - nie obslugiwane jeszcze przez tlen
	STYLE_ADVANCE = TLEN_OPTIONS_PAGEDEF_FLAG_ADVANCEDOPT,	// pozycja dla trybu zaawansowanego opcji - nie obslugiwane jeszcze przez tlen
};


TLENEK_DLLIMPEXP extern const long DEFAULT_OPTIONPAGESTYLE;


class TLENEK_DLLIMPEXP OptionPage : private TlenOptionPageDefinition {

	// to co mozna upychamy do TlenOptionPageDefinition
	private:
		// dane potrzebne w callbacku
		char* m_title;
		char* m_desc;
		HBITMAP m_icon;
		COLORREF m_bgcolor;
		HWND m_hwnd;

		void Init();


	public:

		OptionPage() {
			Init();
		}

		OptionPage(const char* id, const char* name, long style = DEFAULT_OPTIONPAGESTYLE);

		~OptionPage();


		const char* GetId() const {
			return ID;
		}

		void SetId(const char* id) {
			my_cstrdup(ID, id);
		}


		// nie obslugiwane jeszcze przez tlen
		const char* GetParentId() const {
			return ParentID;
		}

		void SetParentId(const char* id) {
			my_cstrdup(ParentID, id);
		}


		const char* GetName() const {
			return Caption;
		}

		void SetName(const char* name) {
			my_cstrdup(Caption, name);
		}


		long GetStyle() {
			return Flags;
		}

		void SetStyle(long style) {
			Flags = style;
		}


		// nie obslugiwane jeszcze przez tlen
		int GetPosition() const {
			return Position;
		}

		void SetPosition(int pos) {
			Position = pos;
		}


		EventHandler* GetEventHandler() {
			return (EventHandler*) HandlerTable::GetInstance()->Get(CallBack);
		}

		void SetEventHandler(EventHandler* handler) {
			CallBack = handler->GetFunction();
		}


		TLENFUNCTION GetCallback() {
			return CallBack;
		}

		void SetCallback(TLENFUNCTION callback) {
			CallBack = callback;
		}


		// ikonka duza na zakladce
		const HBITMAP GetIcon() {
			return m_icon;
		}

		void SetIcons(HBITMAP ico_small, HBITMAP ico_big) {
			Icon = ico_small;
			m_icon = ico_big;
		}

		void SetIcons(const Bitmap* ico_small, const Bitmap* ico_big) {
			Icon = ico_small->Get();
			m_icon = ico_big->Get();
		}


		// tytul okna, max 256 znakow
		const char* GetTitle() const {
			return m_title;
		}

		void SetTitle(const char* title) {
			my_cstrdup(m_title, title);
		}

		// opis pod tytulem, max 1024 znaki
		const char* GetDescription() const {
			return m_desc;
		}

		void SetDescription(const char* desc) {
			my_cstrdup(m_desc, desc);
		}


		COLORREF GetBgColor() {
			return m_bgcolor;
		}

		void SetBgColor(COLORREF color) {
			m_bgcolor = color;
		}


		HWND GetHwnd() {
			return m_hwnd;
		}

		void SetHwnd(HWND hwnd) {
			m_hwnd = hwnd;
		}

};


class TLENEK_DLLIMPEXP OptionsWindow {

	private:

		OptionsWindow() {}
		~OptionsWindow() {}

	public:

		// dodaje zakladke pluginu do okna opcji komunikatora
		void AddPage(const OptionPage* page) {
			Tlen::Get()->GetFunction()->CallTS(TLEN_ADD_OPTIONS_PAGE, (WPARAM) GetPluginInstance(), (LPARAM) page);	

		}

		// usuwa wszystkie zakladki pluginu
		void DelPages() {
			Tlen::Get()->GetFunction()->CallTS(TLEN_REMOVE_OPTIONS_PAGES, (WPARAM) GetPluginInstance(), NULL);
		}


	friend class Tlen;

};


} // namespace tlenek


#endif // _TLENEK_OPTIONS_WINDOW_H_
