/* ==========================================================================
 * Name:        tlenek\plugin.h
 * Purpose:     Deklaracja klasy bazowej Plugin reprezentujacej wtyczke
 *              oraz kilka niezbednych makr dla prostego uzyci biblioteki
 * Author:      Marcin 'malcom' Malich <me@malcom.pl>
 * Modified by:
 * Created:     20/09/2007 15:45:37
 * SVN-ID:      $Id: plugin.h 18 2007-10-19 15:08:56Z malcom $
 * Copyright:   (c) 2007 Marcin 'malcom' Malich <me@malcom.pl>, Poland
 * Licence:     GNU Lesser General Public License
 * ========================================================================== */

#ifndef _TLENEK_PLUGIN_H_
#define _TLENEK_PLUGIN_H_

#include "tlenek\defs.h"
#include "tlenek\tlen.h"
#include "tlenek\utils.h"


// forward declaration
class CPlugApp;
extern BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved);
extern "C" __declspec(dllexport) TLENPLUGININFO* GetPluginInfo(DWORD TlenVersion);
extern "C" __declspec(dllexport) int LoadPlugin(TLENPLUGINFUNCTIONS* tlen_functions);
extern "C" __declspec(dllexport) int UnloadPlugin(void);


namespace tlenek {


// forward declaration from info.h
namespace info {
	DWORD GetTlenVersion();
}


class TLENEK_DLLIMPEXP Plugin {

	private:

		static Plugin* instance;

		HINSTANCE m_hInst;
		TLENPLUGINFUNCTIONS* m_tlen_func;
		TLENPLUGININFO m_plug_info;
		DWORD m_tlen_ver;


	public:

		static Plugin* GetInstance() {
			return instance;
		}

		Plugin();

		~Plugin();


		void SetPluginVersion(int major, int minor, int revision, int build) {
			m_plug_info.PluginVersion = MAKE_DWORD_VERSION(major,minor,revision,build);
		}

		void SetPluginInfo(const char* name, const char* description, const char* copyright) {
			m_plug_info.PluginName = (char*) name;
			m_plug_info.PluginDescription = (char*) description;
			m_plug_info.PluginCopyright = (char*) copyright;
		}

		void SetPluginAutor(const char* author, const char* email, const char* www) {
			m_plug_info.PluginAuthor = (char*) author;
			m_plug_info.PluginAuthorEmail = (char*) email;
			m_plug_info.PluginAuthorHomepage = (char*) www;
		}


		DWORD GetPluginVersion() {
			return m_plug_info.PluginVersion;
		}

		const char* GetPluginName() {
			return m_plug_info.PluginName;
		}

		const char* GetPluginAuthor() {
			return m_plug_info.PluginAuthor;
		}


		HINSTANCE GetPluginInstance() const {
			return m_hInst;
		}

		TLENPLUGINFUNCTIONS* GetTlenFunctions() const {
			return m_tlen_func;
		}

		TLENPLUGININFO* GetPluginInfo() {
			if (OnPluginInfo() != 0) return NULL;
			return &m_plug_info;
		}


		Tlen* GetTlen() {
			return Tlen::Get();
		}


	private:
		
		TLENPLUGININFO* __GetPluginInfo() {
			if (OnPluginInfo() != 0) return NULL;
			return &m_plug_info;
		}

		int __LoadPlugin() { 
			return OnLoad();
		}
		
		int __UnloadPlugin() {
			return OnUnload();
		}


	public:
		
		virtual int OnPluginInfo() { return 0; }
		virtual int OnLoad() { return 0; }
		virtual int OnUnload() { return 0; }


	friend DWORD info::GetTlenVersion();

	friend class ::CPlugApp;
	friend BOOL WINAPI ::DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved);
	friend TLENPLUGININFO* ::GetPluginInfo(DWORD TlenVersion);
	friend int ::LoadPlugin(TLENPLUGINFUNCTIONS* tlen_functions);
	friend int ::UnloadPlugin(void);

};


} // namespace tlenek



#define DECLARE_PLUG(plugname) extern plugname* GetPlug();


#define IMPLEMENT_PLUG(plugname) plugname PlugApp;								\
																				\
plugname * GetPlug() {															\
	return (plugname *) Plugin::GetInstance();									\
}


#define IMPLEMENT_DLL_MFC														\
class CPlugApp : public CWinApp {												\
																				\
	public:																		\
																				\
		BOOL InitInstance() {													\
																				\
			if (!AfxSocketInit()) {												\
				AfxMessageBox("Windows sockets initialization failed.");		\
				return FALSE;													\
			}																	\
																				\
			PlugApp.m_hInst = m_hInstance;										\
			return TRUE;														\
		}																		\
																				\
};																				\
																				\
CPlugApp theApp;


#define IMPLEMENT_DLL_MAIN														\
BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved) {	\
	PlugApp.m_hInst = hinstDLL;													\
	return TRUE;																\
}


#define IMPLEMENT_PLUGIN_FUNCTIONS												\
__declspec(dllexport) TLENPLUGININFO* GetPluginInfo(DWORD tlen_ver) {			\
	TLENEK_AFX																	\
	PlugApp.m_tlen_ver = tlen_ver;												\
	return PlugApp.__GetPluginInfo();											\
}																				\
																				\
__declspec(dllexport) int LoadPlugin(TLENPLUGINFUNCTIONS* functions) {			\
	TLENEK_AFX																	\
	PlugApp.m_tlen_func = functions;											\
	return PlugApp.__LoadPlugin();												\
}																				\
																				\
__declspec(dllexport) int UnloadPlugin(void) {									\
	TLENEK_AFX																	\
	return PlugApp.__UnloadPlugin();											\
}



#ifdef TLENEK_USE_MFC
	#define IMPLEMENT_DLL IMPLEMENT_DLL_MFC
#else
	#define IMPLEMENT_DLL IMPLEMENT_DLL_MAIN
#endif


#define IMPLEMENT_PLUGIN(plugname)		\
	IMPLEMENT_PLUG(plugname)			\
	IMPLEMENT_DLL						\
	IMPLEMENT_PLUGIN_FUNCTIONS


#endif // _TLENEK_PLUGIN_H_
