/* ==========================================================================
 * Name:        tlenek\protocol.h
 * Purpose:     Deklaracja klasy Protocol zarzadzajacej protokolami
 * Author:      Marcin 'malcom' Malich <me@malcom.pl>
 * Modified by:
 * Created:     08/10/2007 20:55:43
 * SVN-ID:      $Id: protocol.h 16 2007-10-17 20:40:54Z malcom $
 * Copyright:   (c) 2007 Marcin 'malcom' Malich <me@malcom.pl>, Poland
 * Licence:     GNU Lesser General Public License
 * ========================================================================== */

#ifndef _TLENEK_PROTOCOL_H_
#define _TLENEK_PROTOCOL_H_

#include "tlenek\defs.h"
#include "tlenek\tlen.h"
#include "tlenek\function.h"
#include "tlenek\hook.h"
#include "tlenek\protocollist.h"
#include "tlenek\misc.h"
#include "tlenek\utils.h"
#include <list>


namespace tlenek {


class ProtocolInfo;


enum ProtocolEventType {
	// event_data ProtocolRawData*
	EVENT_BEFORERECVRAWDATA = PROTOCOL_EVENT_BEFORERECVRAWDATA,
	EVENT_AFTERRECVRAWDATA = PROTOCOL_EVENT_AFTERRECVRAWDATA,
	EVENT_AFTERPROCESSEDRECVRAWDATA = PROTOCOL_EVENT_AFTERPROCESSEDRECVRAWDATA,
	EVENT_AFTERRECVMESSAGE = PROTOCOL_EVENT_AFTERRECVMESSAGE,
	EVENT_BEFORESENDRAWDATA = PROTOCOL_EVENT_BEFORESENDRAWDATA,
	EVENT_AFTERSENDRAWDATA = PROTOCOL_EVENT_AFTERSENDRAWDATA,

	// event_data ContactInfo*
	EVENT_LOGGEDIN = PROTOCOL_EVENT_LOGGEDIN,
	EVENT_LOGGEDOUT = PROTOCOL_EVENT_LOGGEDOUT,

	// event_data ProtocolMessage*
	EVENT_RECVMESSAGE = PROTOCOL_EVENT_RECVMESSAGE,
	EVENT_BEFORESENDMESSAGE = PROTOCOL_EVENT_BEFORESENDMESSAGE,

	// event_data ProtocolPresence*
	EVENT_RECVPRESENCE = PROTOCOL_EVENT_RECVPRESENCE,
	EVENT_BEFORESENDPRESENCE = PROTOCOL_EVENT_BEFORESTATUSCHANGE,

	// event_data ProtocolAuthRequest*
	EVENT_AUTHREQUEST = PROTOCOL_EVENT_AUTHREQUEST,

	// event_data NULL
	EVENT_RECVROSTER = PROTOCOL_EVENT_RECVROSTER,

	// event_data ContactInfo*
	EVENT_CONTACTDELETED = PROTOCOL_EVENT_CONTACTDELETED,

	// event_data ProtocolContactChanged*
	EVENT_CONTACTCHANGED = PROTOCOL_EVENT_CONTACTCHANGED,
};


class TLENEK_DLLIMPEXP Protocol {

	private:

		struct en_t {
			char* proto;

			en_t() {}

			en_t(const en_t& entry) {
				my_strdup(proto, entry.proto, false);
			}
			
			~en_t() {
				delete [] proto;
			}

			int operator==(const en_t& entry) {
				return strcmp(proto, entry.proto) == 0;
			}

		};

		typedef std::list<en_t> en_list_t;
		en_list_t m_list;

		ProtocolList m_pl;

		Protocol() {}
		~Protocol() {}


	public:

		void Add(const ProtocolInfo& proto);

		void Add(const char* id, const char* name, const char* ext = NULL, const char* desc = NULL);

		void Del(const char* id);

		void DelAll();


		// alias do Function::CallProto()
		int Call(const char* proto, const char* name, WPARAM wParam, LPARAM lParam) {
			return Tlen::Get()->GetFunction()->CallProto(proto, name, wParam, lParam);
		}

		// alias do Hook::Notify()
		void Notify(const char* proto, ProtocolEventType event_type, void* even_data) {
			Tlen::Get()->GetHook()->NotifyProto(proto, event_type, even_data);
		}


		int Count() {
			return m_list.size();
		}


		ProtocolList* GetProtocolList(bool only_net = false) {
			m_pl.Update(only_net);
			return &m_pl;
		}


	friend class Tlen;

};


} // namespace tlenek


#endif // _TLENEK_PROTOCOL_H_
