/* ==========================================================================
 * Name:        tlenek\protocols.h
 * Purpose:     Deklaracje klas zwiazanych z eventami protokolow
 * Author:      Marcin 'malcom' Malich <me@malcom.pl>
 * Modified by:
 * Created:     11/10/2007 22:38:24
 * SVN-ID:      $Id: protocols.h 16 2007-10-17 20:40:54Z malcom $
 * Copyright:   (c) 2007 Marcin 'malcom' Malich <me@malcom.pl>, Poland
 * Licence:     GNU Lesser General Public License
 * ========================================================================== */

#ifndef _TLENEK_PROTOCOLS_H_
#define _TLENEK_PROTOCOLS_H_

#include "tlenek\defs.h"
#include "tlenek\contactinfo.h"
#include "tlenek\utils.h"
#include "tlen_sdk\plugin_protocol_defs.h"


namespace tlenek {


enum MessageFlag {
	FLAGS_INCOMING			= PROTOCOL_MESSAGE_FLAGS_INCOMING,
	FLAGS_OUTGOING			= PROTOCOL_MESSAGE_FLAGS_OUTGOING,
	FLAGS_MULTIPLECONTACTS	= PROTOCOL_MESSAGE_FLAGS_MULTIPLECONTACTS
};


class TLENEK_DLLIMPEXP ProtocolMessage : private ProtocolMessageDef {

	public:

		ProtocolMessage() {
			memset(&structSize, 0, sizeof(ProtocolMessageDef));
			structSize = sizeof(ProtocolMessageDef);
		}

		ProtocolMessage(const ProtocolMessage& msg);

		~ProtocolMessage();


		ContactInfo* GetContact() const {
			return (ContactInfo*) &Contact;
		}

		void SetContact(const ContactInfo& contact) {
			CopyContactDef(&Contact, &contact);
		}

		ContactInfoList* GetContacts() const {
			return (ContactInfoList*) &MultipleContacts;
		}

		void SetContacts(const ContactInfoList& contacts) {
			CopyContactsListDef(&MultipleContacts, &contacts);
		}


		enum MessageType {
			OFFLINE		= PROTOCOL_MESSAGE_FLAGS_OFFLINE,
			NORMAL		= PROTOCOL_MESSAGE_FLAGS_NORMAL,
			CHAT		= PROTOCOL_MESSAGE_FLAGS_CHAT,
			WWW			= PROTOCOL_MESSAGE_FLAGS_WWW,
			CONFERENCE	= PROTOCOL_MESSAGE_FLAGS_CONFERENCE
		};

		MessageType GetType() const {
			return (MessageType) (Flags & ~0xFFFFFF00);
		}

		void SetType(MessageType type) {
			// usuwamy flagi poprzedniego typu i ustawiamy nowy
			Flags = (Flags & ~0xFF) | type;
		}


		long GetFlags() const {
			return Flags;
		}

		void SetFlags(long flags) {
			Flags = flags;
		}


		long GetTime() const {
			return TimeStamp;
		}

		void SetTime(long timestamp) {
			TimeStamp = timestamp;
		}


		const char* GetId() const {
			return MessageID;
		}

		void SetId(const char* id) {
			my_cstrdup(MessageID, id);
		}
	

		const char* GetText() const {
			return MessageID;
		}

		void SetText(const char* text) {
			my_cstrdup(MessageText, text);
		}


		// dla konferencji
		const char* GetRoom() const {
			return RoomName;
		}

		void SetRoom(const char* name) {
			my_cstrdup(RoomName, name);
		}


		// dla wiadomosci wyslanych z bramki www
		const char* GetUser() const {
			return UserName;
		}

		void SetUser(const char* user) {
			my_cstrdup(UserName, user);
		}

		const char* GetEmail() const {
			return UserEmail;
		}

		void Setemail(const char* email) {
			my_cstrdup(UserEmail, email);
		}

		const char* GetWww() const {
			return WWWGate;
		}

		void SetWww(const char* www) {
			my_cstrdup(WWWGate, www);
		}


		int GetHandle() const {
			return MessageHandle;
		}

		void SetHandle(int handle) {
			MessageHandle = handle;
		}


		void* GetClientData() const {
			return OwnerData;
		}

		void SetClientData(void* data) {
			OwnerData = data;
		}

};



class TLENEK_DLLIMPEXP ProtocolRawData : private ProtocolRawDataDef {

	public:

		ProtocolRawData() {
			memset(&structSize, 0, sizeof(ProtocolRawDataDef));
			structSize = sizeof(ProtocolRawDataDef);
		}

		ProtocolRawData(const ProtocolRawData& rawdata);

		~ProtocolRawData() {}


		unsigned int GetLength() const {
			return DataLength;
		}

		void SetLength(unsigned int length) {
			DataLength = length;
		}


		const char* GetData() const {
			return Data;
		}

		// Uwaga: dane nie sa kopiowane do wewnetrznego bufora!
		void SetData(const char* data) {
			Data = (char*) data;
		}


		void* GetClientData() const {
			return OwnerData;
		}

		void SetClientData(void* data) {
			OwnerData = data;
		}


		long GetFlags() const {
			return Flags;
		}

		void SetFlags(long flags) {
			Flags = flags;
		}

};



class TLENEK_DLLIMPEXP ProtocolAuthRequest : private ProtocolAuthRequestDef {

	public:

		ProtocolAuthRequest() {
			memset(&structSize, 0, sizeof(ProtocolAuthRequestDef));
			structSize = sizeof(ProtocolAuthRequestDef);
		}

		ProtocolAuthRequest(const ProtocolAuthRequest& request);

		~ProtocolAuthRequest() {
			FreeContactDef(&Contact);
			free(Reason);
		}


		ContactInfo* GetContact() const {
			return (ContactInfo*) &Contact;
		}

		void SetContact(const ContactInfo& contact) {
			CopyContactDef(&Contact, &contact);
		}


		long GetFlags() const {
			return Flags;
		}

		void SetFlags(long flags) {
			Flags = flags;
		}


		const char* GetReason() const {
			return Reason;
		}

		void SetReason(const char* reason) {
			my_cstrdup(Reason, reason);
		}


		long GetTime() const {
			return TimeStamp;
		}

		void SetTime(long timestamp) {
			TimeStamp = timestamp;
		}


		void* GetClientData() const {
			return OwnerData;
		}

		void SetClientData(void* data) {
			OwnerData = data;
		}

};



enum PresenceFlag {
	FLAG_ONLYFRIENDS		= PROTOCOL_PRESENCE_FLAG_ONLYFRIENDS,
	FLAG_AUTOLOGIN			= PROTOCOL_PRESENCE_FLAG_AUTOLOGIN,
	FLAG_AUTOAWAYENTER		= PROTOCOL_PRESENCE_FLAG_AUTOAWAYENTER,
	FLAG_AUTOAWAYEXITKEY	= PROTOCOL_PRESENCE_FLAG_AUTOAWAYEXITKEY,
	FLAG_AUTOAWAYEXITMOUSE	= PROTOCOL_PRESENCE_FLAG_AUTOAWAYEXITMOUSE,
	FLAG_CUSTOMTEXT			= PROTOCOL_PRESENCE_FLAG_CUSTOMTEXT,
	FLAG_PLUGINCHANGED		= PROTOCOL_PRESENCE_FLAG_PLUGINCHANGED,
	FLAG_MENUCLICKED		= PROTOCOL_PRESENCE_FLAG_MENUCLICKED
};


class TLENEK_DLLIMPEXP ProtocolPresence : private ProtocolPresenceDef {

	public:

		ProtocolPresence() {
			memset(&structSize, 0, sizeof(ProtocolPresenceDef));
			structSize = sizeof(ProtocolPresenceDef);
		}

		ProtocolPresence(const ProtocolPresence& presence);

		~ProtocolPresence() {
			FreeContactDef(&Contact);
			free(CurrentDescription);
			free(PreviousDescription);
		}


		ContactInfo* GetContact() const {
			return (ContactInfo*) &Contact;
		}

		void SetContact(const ContactInfo& contact) {
			CopyContactDef(&Contact, &contact);
		}


		enum PresenceState {
			OFFLINE = 1,
			ONLINE,
            CHAT,
			DND,
			AWAY,
			XA,
			INVISIBLE
		};

		PresenceState GetCurState() const {
			return (PresenceState) CurrentState;
		}

		void SetCurState(PresenceState state) {
			CurrentState = state;
		}

		PresenceState GetPrevState() const {
			return (PresenceState) PreviousState;
		}

		void SetPrevState(PresenceState state) {
			PreviousState = state;
		}


		const char* GetCurDescription() const {
			return CurrentDescription;
		}

		void SetCurDescription(const char* desc) {
			my_cstrdup(CurrentDescription, desc);
		}

		const char* GetPrevDescription() const {
			return PreviousDescription;
		}

		void SetPrevDescription(const char* desc) {
			my_cstrdup(PreviousDescription, desc);
		}


		long GetFlags() const {
			return Flags;
		}

		void SetFlags(long flags) {
			Flags = flags;
		}


		long GetTime() const {
			return TimeStamp;
		}

		void SetTime(long timestamp) {
			TimeStamp = timestamp;
		}


		long GetCurComeBackTime() const {
			return CurrentComeBackTimeStamp;
		}

		void SetCurComeBackTime(long timestamp) {
			CurrentComeBackTimeStamp = timestamp;
		}

		long GetPrevComeBackTime() const {
			return PreviousComeBackTimeStamp;
		}

		void SetPrevComeBackTime(long timestamp) {
			PreviousComeBackTimeStamp = timestamp;
		}


		void* GetClientData() const {
			return OwnerData;
		}

		void SetClientData(void* data) {
			OwnerData = data;
		}

};



class TLENEK_DLLIMPEXP ProtocolContactChanged : private ProtocolEventContactChanged {

	public:

		ProtocolContactChanged() {
			memset(&structSize, 0, sizeof(ProtocolEventContactChanged));
			structSize = sizeof(ProtocolEventContactChanged);
		}

		ProtocolContactChanged(const ContactInfo& old_ci, const ContactInfo& new_ci) {
			SetOldContact(old_ci);
			SetNewContact(new_ci);
		}

		ProtocolContactChanged(const ProtocolContactChanged& changed) {
			structSize = changed.structSize;
			CopyContactDef(&old_ct, &changed.old_ct);
			CopyContactDef(&new_ct, &changed.new_ct);
		}

		~ProtocolContactChanged() {
			FreeContactDef(&old_ct);
			FreeContactDef(&new_ct);
		}


		ContactInfo* GetOldContact() const {
			return (ContactInfo*) &old_ct;
		}

		void SetOldContact(const ContactInfo& ci) {
			CopyContactDef(&old_ct, &ci);
		}


		ContactInfo* GetNewContact() const {
			return (ContactInfo*) &new_ct;
		}

		void SetNewContact(const ContactInfo& ci) {
			CopyContactDef(&new_ct, &ci);
		}

};


} // namespace tlenek


#endif // _TLENEK_PROTOCOLS_H_
