/* ==========================================================================
 * Name:        tlenek\stringlist.h
 * Purpose:     Deklaracje klas StringList
 * Author:      Marcin 'malcom' Malich <me@malcom.pl>
 * Modified by:
 * Created:     09/10/2007 12:40:08
 * SVN-ID:      $Id: stringlist.h 15 2007-10-16 14:40:16Z malcom $
 * Copyright:   (c) 2007 Marcin 'malcom' Malich <me@malcom.pl>, Poland
 * Licence:     GNU Lesser General Public License
 * ========================================================================== */

#ifndef _TLENEK_STRING_LIST_H_
#define _TLENEK_STRING_LIST_H_

#include "tlenek\defs.h"
#include "tlen_sdk\sl_struct.h"


namespace tlenek {


// implementacja listy stringow opiera sie na tlenowej stringliscie,
// ktora nie jest zbyt optymalnie i wydajnie napisana,
// przy kazdej operacji dodawania nastepuje reallokacja danych
// a przy realloc() nawet nie jest sprawdzany wynik :/

class TLENEK_DLLIMPEXP StringList : private StringListDef {

	private:

		static bool proc;				// czy pobrano adresy funkcji z tlena

		static sl_init_type       sl_init;
		static sl_add_type        sl_add;
		static sl_delete_type     sl_delete;
		static sl_get_type        sl_get;
		static sl_find_type       sl_find;
		static sl_free_type       sl_free;
		static sl_addstrings_type sl_addstrings;
		static sl_commonpart_type sl_commonpart;

		void Init();


	public:

		StringList() {
			Init();
		}

		StringList(const StringList& sl) {
			Init();
			Merge(sl);
		}

		~StringList() {
			sl_free(this);
		}


		void Add(const char* string) {
			sl_add(this, (char*) string);
		}

		void Del(int index);

		// usuwa pozycje od start do end, lacznie z start, end pozostawia
		void Del(int start, int end);

		// usuwa wszystkie wystapienia ciagu string
		void Del(const char* string);

		void Clear() {
			sl_free(this);
		}		

		const char* Get(int index) const;

		// zwraca index pierwszego wystapienia ciagu string w liscie
		// lub -1 jesli nie znaleziono
		int Find(const char* string) const {
			return sl_find((StringListDef*) this, (char*) string);
		}


		// usuwa z listy pozycje, ktore nie znajduja sie na liscie sl
		void CommonPart(const StringList& sl) {
			sl_commonpart(this, (StringListDef*) &sl);
		}

		// laczy 2 listy
		void Merge(const StringList& sl) {
			sl_addstrings(this, (StringListDef*) &sl);
		}


		int Size() const {
			return count;
		}

		bool Empty() const {
			return count == 0;
		}


		const char* operator[](int index) const {
			return Get(index);
		}

};


} // namespace tlenek


#endif // _TLENEK_STRING_LIST_H_
