/* ==========================================================================
 * Name:        tlenek\xml.h
 * Purpose:     Deklaracja klas parsera XML wykorzystujacego libexpat.dll
 * Author:      Marcin 'malcom' Malich <me@malcom.pl>
 * Modified by:
 * Created:     16/11/2007  18:42:37
 * SVN-ID:      $Id: xml.h 23 2007-11-19 22:05:03Z malcom $
 * Copyright:   (c) 2007 Marcin 'malcom' Malich <me@malcom.pl>, Poland
 * Licence:     GNU Lesser General Public License
 * ========================================================================== */

#ifndef _TLENEK_XML_H_
#define _TLENEK_XML_H_

#include "tlenek\defs.h"
#include "tlenek\info.h"
#include "tlenek\utils.h"
#include <iostream>
#include <fstream>


namespace tlenek {
namespace xml {


/* from expat */

struct XML_ParserStruct;
typedef struct XML_ParserStruct* XML_Parser;

typedef char XML_Char;
typedef char XML_LChar;

/* Should this be defined using stdbool.h when C99 is available? */
typedef unsigned char XML_Bool;
#define XML_TRUE	((XML_Bool) 1)
#define XML_FALSE	((XML_Bool) 0)

enum XML_Error {
	XML_ERROR_NONE,
	XML_ERROR_NO_MEMORY,
	XML_ERROR_SYNTAX,
	XML_ERROR_NO_ELEMENTS,
	XML_ERROR_INVALID_TOKEN,
	XML_ERROR_UNCLOSED_TOKEN,
	XML_ERROR_PARTIAL_CHAR,
	XML_ERROR_TAG_MISMATCH,
	XML_ERROR_DUPLICATE_ATTRIBUTE,
	XML_ERROR_JUNK_AFTER_DOC_ELEMENT,
	XML_ERROR_PARAM_ENTITY_REF,
	XML_ERROR_UNDEFINED_ENTITY,
	XML_ERROR_RECURSIVE_ENTITY_REF,
	XML_ERROR_ASYNC_ENTITY,
	XML_ERROR_BAD_CHAR_REF,
	XML_ERROR_BINARY_ENTITY_REF,
	XML_ERROR_ATTRIBUTE_EXTERNAL_ENTITY_REF,
	XML_ERROR_MISPLACED_XML_PI,
	XML_ERROR_UNKNOWN_ENCODING,
	XML_ERROR_INCORRECT_ENCODING,
	XML_ERROR_UNCLOSED_CDATA_SECTION,
	XML_ERROR_EXTERNAL_ENTITY_HANDLING,
	XML_ERROR_NOT_STANDALONE,
	XML_ERROR_UNEXPECTED_STATE,
	XML_ERROR_ENTITY_DECLARED_IN_PE,
	XML_ERROR_FEATURE_REQUIRES_XML_DTD,
	XML_ERROR_CANT_CHANGE_FEATURE_ONCE_PARSING
};

enum XML_Content_Type {
	XML_CTYPE_EMPTY = 1,
	XML_CTYPE_ANY,
	XML_CTYPE_MIXED,
	XML_CTYPE_NAME,
	XML_CTYPE_CHOICE,
	XML_CTYPE_SEQ
};

enum XML_Content_Quant {
	XML_CQUANT_NONE,
	XML_CQUANT_OPT,
	XML_CQUANT_REP,
	XML_CQUANT_PLUS
};

enum XML_Status {
	XML_STATUS_ERROR = 0,
#define XML_STATUS_ERROR XML_STATUS_ERROR
	XML_STATUS_OK = 1
#define XML_STATUS_OK XML_STATUS_OK
};

typedef struct XML_cp XML_Content;

struct XML_cp {
	enum XML_Content_Type type;
	enum XML_Content_Quant quant;
	XML_Char* name;
	unsigned int numchildren;
	XML_Content* children;
};

typedef struct {
	void* (*malloc_fcn)(size_t size);
	void* (*realloc_fcn)(void* ptr, size_t size);
	void  (*free_fcn)(void* ptr);
} XML_Memory_Handling_Suite;

typedef struct {
	int map[256];
	void* data;
	int  (*convert)(void* data, const char* s);
	void (*release)(void* data);
} XML_Encoding;

enum XML_ParamEntityParsing {
	XML_PARAM_ENTITY_PARSING_NEVER,
	XML_PARAM_ENTITY_PARSING_UNLESS_STANDALONE,
	XML_PARAM_ENTITY_PARSING_ALWAYS
};

typedef struct {
	int major;
	int minor;
	int micro;
} XML_Expat_Version;

enum XML_FeatureEnum {
	XML_FEATURE_END = 0,
	XML_FEATURE_UNICODE,
	XML_FEATURE_UNICODE_WCHAR_T,
	XML_FEATURE_DTD,
	XML_FEATURE_CONTEXT_BYTES,
	XML_FEATURE_MIN_SIZE,
	XML_FEATURE_SIZEOF_XML_CHAR,
	XML_FEATURE_SIZEOF_XML_LCHAR
	/* Additional features must be added to the end of this enum. */
};

typedef struct {
	enum XML_FeatureEnum feature;
	const XML_LChar* name;
	long int value;
} XML_Feature;


typedef void (*XML_AttlistDeclHandler) (void* userData, const XML_Char* elname, const XML_Char* attname, const XML_Char* att_type, const XML_Char* dflt, int isrequired);
typedef void (*XML_StartCdataSectionHandler)(void* userData);
typedef void (*XML_CharacterDataHandler)(void* userData, const XML_Char* s, int len);
typedef void (*XML_CommentHandler)(void* userData, const XML_Char* data);
typedef void (*XML_DefaultHandler)(void* userData, const XML_Char* s, int len);
typedef void (*XML_StartDoctypeDeclHandler)(void* userData, const XML_Char* doctypeName,  const XML_Char* sysid, const XML_Char* pubid, int has_internal_subset);
typedef void (*XML_ElementDeclHandler)(void* userData, const XML_Char* name, XML_Content* model);
typedef void (*XML_EndCdataSectionHandler)(void* userData);
typedef void (*XML_EndDoctypeDeclHandler)(void* userData);
typedef void (*XML_StartElementHandler)(void* userData, const XML_Char* name, const XML_Char** atts);
typedef void (*XML_EndCdataSectionHandler)(void* userData);
typedef void (*XML_EndDoctypeDeclHandler)(void* userData);
typedef void (*XML_EndElementHandler)(void* userData, const XML_Char* name);
typedef void (*XML_EndNamespaceDeclHandler)(void* userData, const XML_Char* prefix);
typedef void (*XML_EntityDeclHandler) (void* userData, const XML_Char* entityName, int is_parameter_entity, const XML_Char* value, int value_length, const XML_Char* base, const XML_Char* systemId, const XML_Char* publicId, const XML_Char* notationName);
typedef int  (*XML_ExternalEntityRefHandler)(XML_Parser parser, const XML_Char* context, const XML_Char* base, const XML_Char* systemId, const XML_Char* publicId);
typedef void (*XML_StartNamespaceDeclHandler)(void* userData, const XML_Char* prefix, const XML_Char* uri);
typedef int  (*XML_NotStandaloneHandler)(void* userData);
typedef void (*XML_NotationDeclHandler)(void* userData, const XML_Char* notationName, const XML_Char* base, const XML_Char* systemId, const XML_Char* publicId);
typedef void (*XML_ProcessingInstructionHandler)(void* userData, const XML_Char* target, const XML_Char* data);
typedef void (*XML_StartNamespaceDeclHandler)(void* userData, const XML_Char* prefix, const XML_Char* uri);
typedef int  (*XML_UnknownEncodingHandler)(void* encodingHandlerData, const XML_Char* name, XML_Encoding* info);
typedef void (*XML_UnparsedEntityDeclHandler)(void* userData, const XML_Char* entityName, const XML_Char* base, const XML_Char* systemId, const XML_Char* publicId, const XML_Char* notationName);
typedef void (*XML_XmlDeclHandler)(void* userData, const XML_Char* version, const XML_Char* encoding, int standalone);
typedef void (*XML_SkippedEntityHandler)(void* userData, const XML_Char* entityName, int is_parameter_entity);


/* funkcje eksportowane w libexpat.dll */

typedef void				(*XML_DefaultCurrent_t)(XML_Parser parser);
typedef const XML_LChar*	(*XML_ErrorString_t)(enum XML_Error code);
typedef const XML_LChar*	(*XML_ExpatVersion_t)(void);
typedef XML_Expat_Version	(*XML_ExpatVersionInfo_t)(void);
typedef XML_Parser			(*XML_ExternalEntityParserCreate_t)(XML_Parser parser, const XML_Char* context, const XML_Char* encoding);
typedef const XML_Char*		(*XML_GetBase_t)(XML_Parser parser);
typedef void*				(*XML_GetBuffer_t)(XML_Parser parser, int len);
typedef int					(*XML_GetCurrentByteCount_t)(XML_Parser parser);
typedef long				(*XML_GetCurrentByteIndex_t)(XML_Parser parser);
typedef int					(*XML_GetCurrentLineNumber_t)(XML_Parser parser);
typedef enum XML_Error		(*XML_GetErrorCode_t)(XML_Parser parser);
typedef int					(*XML_GetIdAttributeIndex_t)(XML_Parser parser);
typedef const char*			(*XML_GetInputContext_t)(XML_Parser parser, int* offset, int* size);
typedef int					(*XML_GetSpecifiedAttributeCount_t)(XML_Parser parser);
typedef enum XML_Status		(*XML_Parse_t)(XML_Parser parser, const char* s, int len, int isFinal);
typedef enum XML_Status		(*XML_ParseBuffer_t)(XML_Parser parser, int len, int isFinal);
typedef XML_Parser			(*XML_ParserCreate_t)(const XML_Char* encoding);
typedef XML_Parser			(*XML_ParserCreateNS_t)(const XML_Char* encoding, XML_Char namespaceSeparator);
typedef XML_Parser			(*XML_ParserCreate_MM_t)(const XML_Char* encoding, const XML_Memory_Handling_Suite* memsuite, const XML_Char* namespaceSeparator);
typedef void				(*XML_ParserFree_t)(XML_Parser parser);
typedef void				(*XML_SetAttlistDeclHandler_t)(XML_Parser parser, XML_AttlistDeclHandler attdecl);
typedef enum XML_Status		(*XML_SetBase_t)(XML_Parser parser, const XML_Char* base);
typedef void				(*XML_SetCdataSectionHandler_t)(XML_Parser parser, XML_StartCdataSectionHandler start, XML_EndCdataSectionHandler end);
typedef void				(*XML_SetCharacterDataHandler_t)(XML_Parser parser, XML_CharacterDataHandler handler);
typedef void				(*XML_SetCommentHandler_t)(XML_Parser parser, XML_CommentHandler handler);
typedef void				(*XML_SetDefaultHandler_t)(XML_Parser parser, XML_DefaultHandler handler);
typedef void				(*XML_SetDefaultHandlerExpand_t)(XML_Parser parser, XML_DefaultHandler handler);
typedef void				(*XML_SetDoctypeDeclHandler_t)(XML_Parser parser, XML_StartDoctypeDeclHandler start, XML_EndDoctypeDeclHandler end);
typedef void				(*XML_SetElementDeclHandler_t)(XML_Parser parser, XML_ElementDeclHandler eldecl);
typedef void				(*XML_SetElementHandler_t)(XML_Parser parser, XML_StartElementHandler start, XML_EndElementHandler end);
typedef enum XML_Status		(*XML_SetEncoding_t)(XML_Parser parser, const XML_Char* encoding);
typedef void				(*XML_SetEndCdataSectionHandler_t)(XML_Parser parser, XML_EndCdataSectionHandler end);
typedef void				(*XML_SetEndDoctypeDeclHandler_t)(XML_Parser parser, XML_EndDoctypeDeclHandler end);
typedef void				(*XML_SetEndElementHandler_t)(XML_Parser, XML_EndElementHandler);
typedef void				(*XML_SetEndNamespaceDeclHandler_t)(XML_Parser parser, XML_EndNamespaceDeclHandler end);
typedef void				(*XML_SetEntityDeclHandler_t)(XML_Parser parser, XML_EntityDeclHandler handler);
typedef void				(*XML_SetExternalEntityRefHandler_t)(XML_Parser parser, XML_ExternalEntityRefHandler handler);
typedef void				(*XML_SetExternalEntityRefHandlerArg_t)(XML_Parser, void* arg);
typedef void				(*XML_SetNamespaceDeclHandler_t)(XML_Parser parser, XML_StartNamespaceDeclHandler start, XML_EndNamespaceDeclHandler end);
typedef void				(*XML_SetNotStandaloneHandler_t)(XML_Parser parser, XML_NotStandaloneHandler handler);
typedef void				(*XML_SetNotationDeclHandler_t)(XML_Parser parser, XML_NotationDeclHandler handler);
typedef int					(*XML_SetParamEntityParsing_t)(XML_Parser parser, enum XML_ParamEntityParsing parsing);
typedef void				(*XML_SetProcessingInstructionHandler_t)(XML_Parser parser, XML_ProcessingInstructionHandler handler);
typedef void				(*XML_SetReturnNSTriplet_t)(XML_Parser parser, int do_nst);
typedef void				(*XML_SetStartCdataSectionHandler_t)(XML_Parser parser, XML_StartCdataSectionHandler start);
typedef void				(*XML_SetStartDoctypeDeclHandler_t)(XML_Parser parser, XML_StartDoctypeDeclHandler start);
typedef void				(*XML_SetStartElementHandler_t)(XML_Parser, XML_StartElementHandler);
typedef void				(*XML_SetStartNamespaceDeclHandler_t)(XML_Parser parser, XML_StartNamespaceDeclHandler start);
typedef void				(*XML_SetUnknownEncodingHandler_t)(XML_Parser parser, XML_UnknownEncodingHandler handler, void* encodingHandlerData);
typedef void				(*XML_SetUnparsedEntityDeclHandler_t)(XML_Parser parser, XML_UnparsedEntityDeclHandler handler);
typedef void				(*XML_SetUserData_t)(XML_Parser parser, void* userData);
typedef void				(*XML_SetXmlDeclHandler_t)(XML_Parser parser, XML_XmlDeclHandler xmldecl);
typedef void				(*XML_UseParserAsHandlerArg_t)(XML_Parser parser);
typedef XML_Bool			(*XML_ParserReset_t)(XML_Parser parser, const XML_Char* encoding);
typedef void				(*XML_SetSkippedEntityHandler_t)(XML_Parser parser, XML_SkippedEntityHandler handler);
typedef const XML_Feature*	(*XML_GetFeatureList_t)(void);
typedef enum XML_Error		(*XML_UseForeignDTD_t)(XML_Parser parser, XML_Bool useDTD);
typedef void				(*XML_FreeContentModel_t)(XML_Parser parser, XML_Content* model);
typedef void*				(*XML_MemMalloc_t)(XML_Parser parser, size_t size);
typedef void*				(*XML_MemRealloc_t)(XML_Parser parser, void* ptr, size_t size);
typedef void				(*XML_MemFree_t)(XML_Parser parser, void* ptr);

//XML_StopParser
//XML_ResumeParser
//XML_GetParsingStatus


static HMODULE hModule = NULL;

TLENEK_DLLIMPEXP void Init(const char* dir);

inline void Deinit() {
	FreeLibrary(hModule);
}


/*  Some of the code adapted from wxXml */

// Represents XML node type.
enum NodeType {
	// note: values are synchronized with xmlElementType from libxml
	ELEMENT_NODE		=  1,
	ATTRIBUTE_NODE		=  2,
	TEXT_NODE			=  3,
	CDATA_SECTION_NODE	=  4,
	ENTITY_REF_NODE		=  5,
	ENTITY_NODE			=  6,
	PI_NODE				=  7,
	COMMENT_NODE		=  8,
	DOCUMENT_NODE		=  9,
	DOCUMENT_TYPE_NODE	= 10,
	DOCUMENT_FRAG_NODE	= 11,
	NOTATION_NODE		= 12,
	HTML_DOCUMENT_NODE	= 13
};


class TLENEK_DLLIMPEXP Attribute {

	private:
		char* m_name;
		char* m_value;
		Attribute* m_next;


	public:

		Attribute() {
			m_name = NULL;
			m_value = NULL;
			m_next = NULL;
		}

		Attribute(const char* name, const char* value, Attribute* next = NULL) {
			my_strdup(m_name, name, false);
			my_strdup(m_value, value, false);
			m_next = next;
		}

		~Attribute() {}


		const char* GetName() const {
			return m_name;
		}

		void SetName(const char* name) {
			my_strdup(m_name, name);
		}


		const char* GetValue() const {
			return m_value;
		}

		void SetValue(const char* value) {
			my_strdup(m_value, value);
		}


		Attribute* GetNext() const {
			return m_next;
		}

		void SetNext(Attribute* next) {
			m_next = next;
		}

};


class TLENEK_DLLIMPEXP Node {

	private:

		NodeType m_type;
		char* m_name;
		char* m_content;
		Attribute* m_attrs;
		Node *m_parent, *m_children, *m_next;
		int m_lineNo;		// line number in original file, or -1

		void DoCopy(const Node& node);


	public:

		Node() {
			m_name = NULL;
			m_content = NULL;
			m_attrs = NULL;
			m_parent = NULL;
			m_children = NULL;
			m_next = NULL;
			m_lineNo = -1;
		}

		Node(Node* parent, NodeType type, const char* name, const char* content = NULL,
			 Attribute* attrs = NULL, Node* next = NULL, int lineNo = -1);

		Node(NodeType type, const char* name, const char* content = NULL, int lineNo = -1);

		// copy ctor & operator=. Note that this does NOT copy syblings
		// and parent pointer, i.e. m_parent and m_next will be NULL
		// after using copy ctor and are never unmodified by operator=.
		// On the other hand, it DOES copy children and attributes.

		Node(const Node& node);
		Node& operator=(const Node& node);

		~Node();


		NodeType GetType() const {
			return m_type;
		}

		void SetType(NodeType type) {
			m_type = type;
		}


		const char* GetName() const {
			return m_name;
		}

		void SetName(const char* name) {
			my_strdup(m_name, name);
		}


		const char* GetContent() const {
			return m_content;
		}

		void SetContent(const char* con) {
			my_strdup(m_content, con);
		}


		Node* GetParent() const {
			return m_parent;
		}

		void SetParent(Node* parent) {
			m_parent = parent;
		}

		Node* GetNext() const {
			return m_next;
		}

		void SetNext(Node* next) {
			m_next = next;
		}
		

		// childrens //

		bool HasChild() const {
			return m_children != NULL;
		}

		Node* GetChildren() const {
			return m_children;
		}

		void SetChildren(Node* child) {
			m_children = child;
		}

		
		void AddChild(Node* child);
		bool InsertChild(Node* child, Node* before_node);

		bool RemoveChild(Node* child);


		// attributes //

		bool HasAttrib() const {
			return m_attrs != NULL;
		}

		Attribute* GetAttributes() const {
			return m_attrs;
		}

		void SetAttributes(Attribute* attr) {
			m_attrs = attr;
		}


		void AddAttribute(const char* name, const char* value) {
			AddAttribute(new Attribute(name, value, NULL));
		}
		
		void AddAttribute(Attribute* attr);

		bool GetAttribute(const char* name, char*& value) const;
		const char* GetAttribute(const char* name, const char* defaultVal) const;

		bool DeleteAttribute(const char* name);



		bool IsWhitespaceOnly() const {
			return StrIsWhiteOnly(m_content);
		}
		
		const char* GetNodeContent() const;

		int GetDepth(Node* grandparent = NULL) const;

		int GetLineNumber() const {
			return m_lineNo;
		}

};


class TLENEK_DLLIMPEXP Document {

	private:

		char* m_version;
		char* m_encoding;
		Node* m_root;

		void Init() {
			// w przypadku budowania pluginu na tlenku
			// parser XML automatycznie inicjujemy w razie potrzeby,
			// zwalniany jest w destruktorze pluginu
			if (Plugin::GetInstance() != NULL && hModule == NULL) {
				xml::Init(info::GetTlenPath());
			}
		}

		void DoCopy(const Document& doc);


	public:

		Document();

		Document(const char* filename, const char* encoding = "UTF-8");

		Document(const Document& doc);
		Document& operator=(const Document& doc);

		~Document() {
			delete [] m_version;
			delete [] m_encoding;
			delete m_root;
		}


		bool Load(const char* filename, const char* encoding = "UTF-8", bool keep_wnodes = false);
		bool Save(const char* filename, int indentstep = 1) const;


		bool FromString(const char* str, size_t len);
		bool ToString(char* buf, size_t* len);


		bool IsOk() const {
			return m_root != NULL;
		}


		Node* GetRoot() const {
			return m_root;
		}

		void SetRoot(Node* node) {
			delete m_root;
			m_root = node;
		}


		const char* GetVersion() const {
			return m_version;
		}

		void SetVersion(const char* version) {
			my_strdup(m_version, version);
		}


		const char* GetEncoding() const {
			return m_encoding;
		}

		void SetEncoding(const char* encoding) {
			my_strdup(m_encoding, encoding);
		}


		Node* DetachRoot() {
			Node* old = m_root;
			m_root = NULL;
			return old;
		}


	private:

		bool Load(std::istream& stream, const char* encoding = "UTF-8", bool keep_wnodes = false);
		bool Save(std::ostream& stream, int indentstep = 1) const;

};


} // namespace xml
} // namespace tlenek


#endif // _TLENEK_XML_H_
