/* ==========================================================================
 * Name:        contacticon.cpp
 * Purpose:     Implementacja klasy ContactIcon
 * Author:      Marcin 'malcom' Malich <me@malcom.pl>
 * Modified by:
 * Created:     24/09/2007 04:00:24
 * SVN-ID:      $Id: contacticon.cpp 14 2007-10-15 17:33:59Z malcom $
 * Copyright:   (c) 2007 Marcin 'malcom' Malich <me@malcom.pl>, Poland
 * Licence:     GNU Lesser General Public License
 * ========================================================================== */

#include "tlenek\contacticon.h"
#include "tlenek\image.h"
#include "tlenek\tlen.h"
#include "tlenek\function.h"
#include "tlenek\exception.h"
#include "tlenek\utils.h"


namespace tlenek {


void ContactIcon::Init() {
	memset(&structSize, 0, sizeof(ContactListIconDef));

	// na podstawie structSize bedziemy rozrozniac czy nasz obiekt (structSize=0)
	// czy tlenowy, i na tej podstawie alokowac wiekszy bufor dla naszego obiektu,
	structSize = 0;

	// defaultowo/poczatkowo alokujemy tablice na 10 obrazkow
	SingleIcons = (TlenSingleImageDef*) malloc(sizeof(TlenSingleImageDef) * ImageSizeAlloc);
	if (SingleIcons != NULL) {
		m_image_alloc = ImageSizeAlloc;
	} else {
		m_image_alloc = 0;
	}
}


ContactIcon::~ContactIcon() {
	free(ID);
	free(IconHint);
	for (int i=0; i<StatesCount; i++) {
		FreeTlenSingleImageDef(&SingleIcons[i]);
	}
	free(SingleIcons);
}


void ContactIcon::SetState(int index) {
	if (structSize == 0) {
		CurrentState = index;
		Flags |= CONTACTLIST_ICONFLAG_CHANGE_CURRENTSTATE;

	} else {
		// state = index+1
		if (index > 0 && index <= StatesCount) {
			CurrentState = index;
		} else {
			throw Exception(VALIDINDEX, NULL, "ContactIcon::SetState", __LINE__, __FILE__);
		}
	}
}


void ContactIcon::AddImage(const Image& image) {
	if (structSize == 0) {
		if (StatesCount == m_image_alloc) {
			int size = m_image_alloc + ImageSizeAlloc;
			TlenSingleImageDef* ptr = (TlenSingleImageDef*) realloc(SingleIcons, sizeof(TlenSingleImageDef) * size);
			if (ptr == NULL) return;
			SingleIcons = ptr;
			m_image_alloc = size;
		}
		Flags |= CONTACTLIST_ICONFLAG_CHANGE_SINGLEICONS;

	} else {
		TlenSingleImageDef* ptr = (TlenSingleImageDef*) realloc(SingleIcons, sizeof(TlenSingleImageDef) * (StatesCount+1));
		if (ptr == NULL) return;
		SingleIcons = ptr;
		if (image.GetSkinId()) {
			// jak ikonka skorowalna, to zaladowac jej dane
			Tlen::Get()->GetFunction()->CallTS(TLEN_SKIN_GETSKINNEDIMAGE, (WPARAM) &image, NULL);
		}
	}

	CopyTlenSingleImageDef(&SingleIcons[StatesCount], &image);

	// aktualizacja licznika obrazkow
	StatesCount++;
}


void ContactIcon::DelImage(int index) {
	if (index >= 0 && index < StatesCount) {
		FreeTlenSingleImageDef(&SingleIcons[index]);
		memmove(SingleIcons + index, SingleIcons + index + 1, sizeof(TlenSingleImageDef)*(--StatesCount - index));
		if (structSize == 0) {
			Flags |= CONTACTLIST_ICONFLAG_CHANGE_SINGLEICONS;
		}
	} else {
		throw Exception(VALIDINDEX, NULL, "ContactIcon::DelImage", __LINE__, __FILE__);
	}
}


Image* ContactIcon::GetImage(int index) {
	if (index >= 0 && index < StatesCount) {
		// jakbysmy pobrali i skopiowali ikone, a pozniej edytowali obrazek ;p
		if (structSize == 0) {
			Flags |= CONTACTLIST_ICONFLAG_CHANGE_SINGLEICONS;
		}
		return (Image*) &SingleIcons[index];
	} else {
		throw Exception(VALIDINDEX, NULL, "ContactIcon::GetImage", __LINE__, __FILE__);
	}
}



void CopyContactListIconDef(ContactListIconDef* dst, const ContactListIconDef* src) {

	dst->structSize = src->structSize;
	my_cstrdup(dst->ID, src->ID, false);
	dst->StatesCount = 0;
	dst->SingleIcons = (TlenSingleImageDef*) malloc(sizeof(TlenSingleImageDef) * src->StatesCount);
	if (dst->SingleIcons != NULL) {
		dst->StatesCount = src->StatesCount;
		for (int i=0; i<dst->StatesCount; i++) {
			CopyTlenSingleImageDef(&dst->SingleIcons[i], &src->SingleIcons[i]);
		}
	}
	dst->IconKind = src->IconKind;
	dst->Flags = src->Flags;
	dst->CurrentState = src->CurrentState;
	my_cstrdup(dst->IconHint, src->IconHint, false);
	dst->CallBackFunc = src->CallBackFunc;
	dst->OwnerData = src->OwnerData;
	CopyRect(&dst->DestRect, &src->DestRect);

}


void FreeContactListIconDef(ContactListIconDef* dst) {

	free(dst->ID);
	for (int i=0; i<dst->StatesCount; i++) {
		FreeTlenSingleImageDef(&dst->SingleIcons[i]);
	}
	free(dst->SingleIcons);
	free(dst->IconHint);

}


} // namespace tlenek

