/* ==========================================================================
 * Name:        contactitem.cpp
 * Purpose:     Implementacje klas elementow listy kontaktow
 * Author:      Marcin 'malcom' Malich <me@malcom.pl>
 * Modified by:
 * Created:     20/09/2007 20:04:10
 * SVN-ID:      $Id: contactitem.cpp 4 2007-10-05 00:35:00Z malcom $
 * Copyright:   (c) 2007 Marcin 'malcom' Malich <me@malcom.pl>, Poland
 * Licence:     GNU Lesser General Public License
 * ========================================================================== */

#include <string.h>
#include "tlenek\contactitem.h"
#include "tlenek\contacticon.h"
#include "tlenek\tlen.h"
#include "tlenek\function.h"
#include "tlenek\exception.h"


namespace tlenek {


const long DEFAULT_CONTACTITEMSTYLE = STYLE_EXPANDABLE | STYLE_CHILDRENVISIBLE;


void ContactItem::Init() {
	// szybkie czyszczenie struct
	// biorac pod uwage ze sizeStruct jest pierwsza pozycja
	// wyczyscimy w ten sposob caly struct ContactListItemDef
	memset(&structSize, 0, sizeof(ContactListItemDef));
	structSize = sizeof(ContactListItemDef);

	// defaultowo/poczatkowo alokujemy tablice na 3 ikonki
	Icons = (ContactListIconDef*) malloc(sizeof(ContactListIconDef) * IconSizeAlloc);
	if (Icons != NULL) {
		m_icon_alloc = IconSizeAlloc;
	} else {
		m_icon_alloc = 0;
	}
}


ContactItem::ContactItem(ContactItemType type, int section, int group, const char* name, 
				long style, ContactList::pos pos) {
	Init();
	SetItemType(type);
	SetSectionId(section);
	SetGroupId(group);
	SetMainText(name);
	SetStyle(style);
	SetPosition(pos);
}


ContactItem::~ContactItem() {
	// zwalniamy tablice tylko jesli jest ona nasza robota
	if (ItemHandle == 0 && m_icon_alloc > 0) {
		// ale najpierw trzeba zwolnic znajdujace sie w niej dane
		for (int i = 0; i < IconsCount; i++) {
			FreeContactListIconDef(&Icons[i]);
		}
		free(Icons);
	}
}


void ContactItem::SetPosition(ContactList::pos pos) {
	if (GetId() == 0) {
		// jeszcze nie dodany element - aktualizujemy pole w naszej struct parenta
		PreviousItemHandle = pos;
		// flaga zmian, potrzebne, gdybysmy chcieli zminic korzystjac z metody
		// ContactList::ChangeItem()/UpdateItem()
		ChangeFlags |= CONTACTLIST_ITEMCHANGEFLAG_PREVIOUSHANDLE;

	} else {
		// element pochodzi z tlena - aktualizujemy bezposrednio w tlenie
		ContactListItemDef def;
		InitializeStruct(def);
		def.PreviousItemHandle = pos;
		def.ChangeFlags = CONTACTLIST_ITEMCHANGEFLAG_PREVIOUSHANDLE;
		// i udate w tlenie, bez odmalowania pozycji i listy
		// bedziemy chcieli to sobie wywolamy metode Redraw()
		// moglibysmy uzyc ContactList::UpdateItem(), ale znow kilka niepotrzebnych
		// narzutow i kodu zawartego w tej metodzie, choc to tylko 1 if ?:|
		Tlen::Get()->GetFunction()->CallTS(TLEN_CONTACTLIST_CHANGEITEM, (WPARAM) ItemHandle, (LPARAM) &def);
	}
}


ContactItem::ContactItemType ContactItem::GetItemType() const {
	switch (ItemKind) {
		case CONTACTLIST_ITEMKIND_SECTION:
			return SECTION;
		case CONTACTLIST_ITEMKIND_GROUP:
			return GROUP;
		case CONTACTLIST_ITEMKIND_ELEMENT:
			switch (ItemSubKind) {
				case CONTACTLIST_ITEMSUBKIND_STATICBAR:
					return BAR;
				case CONTACTLIST_ITEMSUBKIND_CONTACT:
					return CONTACT;
				case CONTACTLIST_ITEMSUBKIND_METAGROUP:
					return META;
				default:
					return OTHER;
			}
		default:
			return OTHER;
	}
}


void ContactItem::SetStyle(long style) {
	if (GetId() == 0) {
		Flags1 = style;
		ChangeFlags |= CONTACTLIST_ITEMCHANGEFLAG_FLAGS1;
	} else {
		ContactListItemDef def;
		InitializeStruct(def);
		def.Flags1 = style;
		def.ChangeFlags = CONTACTLIST_ITEMCHANGEFLAG_FLAGS1;
		Tlen::Get()->GetFunction()->CallTS(TLEN_CONTACTLIST_CHANGEITEM, (WPARAM) ItemHandle, (LPARAM) &def);
	}		
}


void ContactItem::SetMenuId(const char* id) {
	if (GetId() == 0) {
		MenuID = (char*) id;
		ChangeFlags |= CONTACTLIST_ITEMCHANGEFLAG_MENUID;
	} else {
		ContactListItemDef def;
		InitializeStruct(def);
		def.MenuID = (char*) id;
		def.ChangeFlags = CONTACTLIST_ITEMCHANGEFLAG_MENUID;
		Tlen::Get()->GetFunction()->CallTS(TLEN_CONTACTLIST_CHANGEITEM, (WPARAM) ItemHandle, (LPARAM) &def);
	}		
}


void ContactItem::SetEventHandler(EventHandler* handler) {
	if (GetId() == 0) {
		CallBackFunc = handler->GetFunction();
		ChangeFlags |= CONTACTLIST_ITEMCHANGEFLAG_MENUID;
	} else {
		ContactListItemDef def;
		InitializeStruct(def);
		def.CallBackFunc = handler->GetFunction();
		def.ChangeFlags = CONTACTLIST_ITEMCHANGEFLAG_CALLBACKFUNC;
		Tlen::Get()->GetFunction()->CallTS(TLEN_CONTACTLIST_CHANGEITEM, (WPARAM) ItemHandle, (LPARAM) &def);
	}		
}


void ContactItem::SetCallback(TLENFUNCTION callback) {
	if (GetId() == 0) {
		CallBackFunc = callback;
		ChangeFlags |= CONTACTLIST_ITEMCHANGEFLAG_MENUID;
	} else {
		ContactListItemDef def;
		InitializeStruct(def);
		def.CallBackFunc = callback;
		def.ChangeFlags = CONTACTLIST_ITEMCHANGEFLAG_CALLBACKFUNC;
		Tlen::Get()->GetFunction()->CallTS(TLEN_CONTACTLIST_CHANGEITEM, (WPARAM) ItemHandle, (LPARAM) &def);
	}		
}


void ContactItem::SetClientData(void* data) {
	if (GetId() == 0) {
		OwnerData = data;
		ChangeFlags |= CONTACTLIST_ITEMCHANGEFLAG_OWNERDATA;
	} else {
		ContactListItemDef def;
		InitializeStruct(def);
		def.OwnerData = data;
		def.ChangeFlags = CONTACTLIST_ITEMCHANGEFLAG_OWNERDATA;
		Tlen::Get()->GetFunction()->CallTS(TLEN_CONTACTLIST_CHANGEITEM, (WPARAM) ItemHandle, (LPARAM) &def);
	}		
}



void ContactItem::SetItemType(ContactItemType type) {
	int kind, subkind;
	switch (type) {
		case SECTION:
			kind = CONTACTLIST_ITEMKIND_SECTION;
			break;
		case GROUP:
			kind = CONTACTLIST_ITEMKIND_GROUP;
			break;
		case BAR:
			kind = CONTACTLIST_ITEMKIND_ELEMENT;
			subkind = CONTACTLIST_ITEMSUBKIND_STATICBAR;
			break;
		case META:
			kind = CONTACTLIST_ITEMKIND_ELEMENT;
			subkind = CONTACTLIST_ITEMSUBKIND_METAGROUP;
			break;
		case CONTACT:
			kind = CONTACTLIST_ITEMKIND_ELEMENT;
			subkind = CONTACTLIST_ITEMSUBKIND_CONTACT;
			break;
		default:
			kind = CONTACTLIST_ITEMKIND_ELEMENT;
			subkind = CONTACTLIST_ITEMSUBKIND_OTHER;
			break;
	}

	if (GetId() == 0) {
		ItemKind = kind;
		ItemSubKind = subkind;
		ChangeFlags |= CONTACTLIST_ITEMCHANGEFLAG_ITEMKIND || CONTACTLIST_ITEMCHANGEFLAG_ITEMSUBKIND;
	} else {
		ContactListItemDef def;
		InitializeStruct(def);
		def.ItemKind = kind;
		def.ItemSubKind = subkind;
		def.ChangeFlags = CONTACTLIST_ITEMCHANGEFLAG_ITEMKIND || CONTACTLIST_ITEMCHANGEFLAG_ITEMSUBKIND;
		Tlen::Get()->GetFunction()->CallTS(TLEN_CONTACTLIST_CHANGEITEM, (WPARAM) ItemHandle, (LPARAM) &def);
	}
}


void ContactItem::SetFlag(int flag) {
	if (GetId() == 0) {
		Flags1 |= flag;
		ChangeFlags |= CONTACTLIST_ITEMCHANGEFLAG_FLAGS1;
	} else {
		ContactListItemDef def;
		InitializeStruct(def);
		def.Flags1 = Flags1 | flag;
		def.ChangeFlags = CONTACTLIST_ITEMCHANGEFLAG_FLAGS1;
		Tlen::Get()->GetFunction()->CallTS(TLEN_CONTACTLIST_CHANGEITEM, (WPARAM) ItemHandle, (LPARAM) &def);
	}
}


void ContactItem::UnsetFlag(int flag) {
	if (GetId() == 0) {
		Flags1 &= ~flag;
		ChangeFlags |= CONTACTLIST_ITEMCHANGEFLAG_FLAGS1;
	} else {
		ContactListItemDef def;
		InitializeStruct(def);
		def.Flags1 = Flags1 & ~flag;
		def.ChangeFlags = CONTACTLIST_ITEMCHANGEFLAG_FLAGS1;
		Tlen::Get()->GetFunction()->CallTS(TLEN_CONTACTLIST_CHANGEITEM, (WPARAM) ItemHandle, (LPARAM) &def);
	}
}


void ContactItem::SetContactType(const ContactType type) {
	if (GetId() == 0) {
		Flags2 = type;
		ChangeFlags |= CONTACTLIST_ITEMCHANGEFLAG_FLAGS2;
	} else {
		ContactListItemDef def;
		InitializeStruct(def);
		def.Flags2 = type;
		def.ChangeFlags = CONTACTLIST_ITEMCHANGEFLAG_FLAGS2;
		Tlen::Get()->GetFunction()->CallTS(TLEN_CONTACTLIST_CHANGEITEM, (WPARAM) ItemHandle, (LPARAM) &def);
	}
}


void ContactItem::SetSectionId(int id) {
	if (GetId() == 0) {
		SectionHandle = id;
		ChangeFlags |= CONTACTLIST_ITEMCHANGEFLAG_SECTIONHANDLE;
	} else {
		ContactListItemDef def;
		InitializeStruct(def);
		def.SectionHandle = id;
		def.ChangeFlags = CONTACTLIST_ITEMCHANGEFLAG_SECTIONHANDLE;
		Tlen::Get()->GetFunction()->CallTS(TLEN_CONTACTLIST_CHANGEITEM, (WPARAM) ItemHandle, (LPARAM) &def);
	}
}


void ContactItem::SetGroupId(int id) {
	if (GetId() == 0) {
		GroupHandle = id;
		ChangeFlags |= CONTACTLIST_ITEMCHANGEFLAG_GROUPHANDLE;
	} else {
		ContactListItemDef def;
		InitializeStruct(def);
		def.GroupHandle = id;
		def.ChangeFlags = CONTACTLIST_ITEMCHANGEFLAG_GROUPHANDLE;
		Tlen::Get()->GetFunction()->CallTS(TLEN_CONTACTLIST_CHANGEITEM, (WPARAM) ItemHandle, (LPARAM) &def);
	}
}


void ContactItem::SetMetaId(int id) {

	if (GetId() == 0) {
		MetaGroupHandle = id;
//		ChangeFlags |= CONTACTLIST_ITEMCHANGEFLAG_METAHANDLE;
	} else {
//		ContactListItemDef def;
//		InitializeStruct(def);
//		def.MetaGroupHandle = id;
//		def.ChangeFlags = CONTACTLIST_ITEMCHANGEFLAG_METAHANDLE;
//		tlen_functions->CallTlenFunction(hInst, TLEN_CONTACTLIST_CHANGEITEM, (WPARAM) ItemHandle, (LPARAM) &def);
	}
}


void ContactItem::SetMainText(const char* text) {
	if (GetId() == 0) {
		MainText = (char*) text;
		ChangeFlags |= CONTACTLIST_ITEMCHANGEFLAG_MAINTEXT;
	} else {
		ContactListItemDef def;
		InitializeStruct(def);
		def.MainText = (char*) text;
		def.ChangeFlags = CONTACTLIST_ITEMCHANGEFLAG_MAINTEXT;
		Tlen::Get()->GetFunction()->CallTS(TLEN_CONTACTLIST_CHANGEITEM, (WPARAM) ItemHandle, (LPARAM) &def);
	}
}
		

void ContactItem::SetSecondaryText(const char* text) {
	if (GetId() == 0) {
		SecondaryText = (char*) text;
		ChangeFlags |= CONTACTLIST_ITEMCHANGEFLAG_SECONDARYTEXT;
	} else {
		ContactListItemDef def;
		InitializeStruct(def);
		def.SecondaryText = (char*) text;
		def.ChangeFlags = CONTACTLIST_ITEMCHANGEFLAG_SECONDARYTEXT;
		Tlen::Get()->GetFunction()->CallTS(TLEN_CONTACTLIST_CHANGEITEM, (WPARAM) ItemHandle, (LPARAM) &def);
	}
}
		

void ContactItem::SetLowerText(const char* text) {
	if (GetId() == 0) {
		LowerText = (char*) text;
		ChangeFlags |= CONTACTLIST_ITEMCHANGEFLAG_LOWERTEXT;
	} else {
		ContactListItemDef def;
		InitializeStruct(def);
		def.LowerText = (char*) text;
		def.ChangeFlags = CONTACTLIST_ITEMCHANGEFLAG_LOWERTEXT;
		Tlen::Get()->GetFunction()->CallTS(TLEN_CONTACTLIST_CHANGEITEM, (WPARAM) ItemHandle, (LPARAM) &def);
	}
}

		
void ContactItem::SetItemHint(const char* text) {
	if (GetId() == 0) {
		ItemHint = (char*) text;
		ChangeFlags |= CONTACTLIST_ITEMCHANGEFLAG_ITEMHINT;
	} else {
		ContactListItemDef def;
		InitializeStruct(def);
		def.ItemHint = (char*) text;
		def.ChangeFlags = CONTACTLIST_ITEMCHANGEFLAG_ITEMHINT;
		Tlen::Get()->GetFunction()->CallTS(TLEN_CONTACTLIST_CHANGEITEM, (WPARAM) ItemHandle, (LPARAM) &def);
	}
}


void ContactItem::SetItemNewHint(const char* text) {
	if (GetId() == 0) {
		ItemNewHint = (char*) text;
		ChangeFlags |= CONTACTLIST_ITEMCHANGEFLAG_ITEMNEWHINT;
	} else {
		ContactListItemDef def;
		InitializeStruct(def);
		def.ItemNewHint = (char*) text;
		def.ChangeFlags = CONTACTLIST_ITEMCHANGEFLAG_ITEMNEWHINT;
		Tlen::Get()->GetFunction()->CallTS(TLEN_CONTACTLIST_CHANGEITEM, (WPARAM) ItemHandle, (LPARAM) &def);
	}
}


void ContactItem::AddIcon(const ContactIcon& icon) {
	if (GetId() == 0) {
		// element jeszcze nie dodany do listy - ikony pakujemy do tablicy
		// powinnismy chyba sprawdzic czy istnieje juz ikona o takim samym id,
		// ale olejemy to, tlen taka ikone pominie przy dodawaniu
		// tablica pelna, realokacja
		if (IconsCount == m_icon_alloc) {
			int size = m_icon_alloc + IconSizeAlloc;
			ContactListIconDef* ptr = (ContactListIconDef*) realloc(Icons, sizeof(ContactListIconDef) * size);
			if (ptr == NULL) return;
			Icons = ptr;
			m_icon_alloc = size;
		}
		CopyContactListIconDef(&Icons[IconsCount], &icon);

		// aktualizacja licznika ikon
		// bo to on jest zwracany jako ilosc ikon
		IconsCount++;

	} else {
		// element w tlenie - ikony dodajemy bezposrednio do tlena
		// moglibysmy dodac hurtem kilka ikon przy aktualizacji, ale po co kombinowac ;p
		Tlen::Get()->GetContactList()->AddIcon(ItemHandle, icon);
	}
}


void ContactItem::DelIcon(const char* id) {
	if (id == NULL) return;

	if (GetId() == 0) {
		// jak element nie dodany to usuwanie z tablicy ikonek
		size_t len = strlen(id);

		// sprawdz czy usuwanie wielu ikonek
		if (strstr(id, "***")) {
			len -= 3;
		}

		// najprosciej byloby usunac ikonke i na jej miejsce
		// przeniesc ostatnia pozycje z tablicy, ale wtedy zburzylibysmy kolejnosc,
		// wiec od konca usuwamy ionki i przesuwamy dane, aby "zalatac" dziure
		for (int i=IconsCount-1; i>=0; i--) {
			if (Icons[i].ID && strncmp(Icons[i].ID, id, len) == 0) {
				FreeContactListIconDef(&Icons[i]);
				memmove(Icons + i, Icons + i + 1, sizeof(ContactListIconDef)*(--IconsCount - i));
			}
		}

	} else {
		// usuwanie bezposrednio z tlena
		Tlen::Get()->GetContactList()->DelIcon(ItemHandle, id);
	}
}


ContactIcon* ContactItem::GetIcon(const char* id) {
	for (int i=0; i<IconsCount; i++) {
		if (strcmp(Icons[i].ID, id) == 0) {
			return (ContactIcon*) &Icons[i];
		}
	}
	return NULL;
}


ContactIcon* ContactItem::GetIcon(int index) {
	if (index >= 0 && index < IconsCount) {
		return (ContactIcon*) &Icons[index];
	} else {
		throw Exception(VALIDINDEX, NULL, "ContactItem::GetIcon", __LINE__, __FILE__);
	}
}


} // namespace tlenek
