/* ==========================================================================
 * Name:        contactlist.cpp
 * Purpose:     Implementacja klasy obslugi listy kontaktow
 * Author:      Marcin 'malcom' Malich <me@malcom.pl>
 * Modified by:
 * Created:     20/09/2007 18:25:27
 * SVN-ID:      $Id: contactlist.cpp 14 2007-10-15 17:33:59Z malcom $
 * Copyright:   (c) 2007 Marcin 'malcom' Malich <me@malcom.pl>, Poland
 * Licence:     GNU Lesser General Public License
 * ========================================================================== */

#include "tlenek\contactlist.h"
#include "tlenek\contactitem.h"
#include "tlenek\contacticon.h"
#include "tlenek\tlen.h"
#include "tlenek\function.h"
#include "tlenek\stringlist.h"


namespace tlenek {


const ContactList::pos ContactList::POS_FIRST = CONTACTLIST_ITEMPOS_FIRST;
const ContactList::pos ContactList::POS_LAST = CONTACTLIST_ITEMPOS_LAST;


ContactList::ContactList() {
	sms_number   = Tlen::Get()->GetFunction()->GetProcAddr(TPA_GET_SMS_NUMBER);
	sms_contact  = Tlen::Get()->GetFunction()->GetProcAddr(TPA_GET_SMS_NUMBER_CONTACT);
	contact_name = Tlen::Get()->GetFunction()->GetProcAddr(TPA_GET_CONTACT_NAME);
	contact_desc = Tlen::Get()->GetFunction()->GetProcAddr(TPA_GET_CONTACT_STATUS_TEXT);
}


int ContactList::AddItem(const ContactItem& item) {
	int ret = Tlen::Get()->GetFunction()->CallTS(TLEN_CONTACTLIST_ADDITEM, (WPARAM) &item, NULL);
	if (!ret) {
		// wyjatek?
	}
	ContactItem* it = (ContactItem*) &item;
	ret = it->ItemHandle;
	it->ItemHandle = 0;
	return ret;
}


void ContactList::DelItem(int id) {
	int ret = Tlen::Get()->GetFunction()->CallTS(TLEN_CONTACTLIST_REMOVEITEM, (WPARAM) id, NULL);
	if (!ret) {
		// wyjatek?
	}
}


void ContactList::UpdateItem(int id, const ContactItem& item, UpdateItemFlag flag) {
	if (flag != NONE) {
		ContactListItemDef* def = (ContactListItemDef*) &item;
		long cflg = def->ChangeFlags;
		def->ChangeFlags |= flag;
		Tlen::Get()->GetFunction()->CallTS(TLEN_CONTACTLIST_CHANGEITEM, (WPARAM) id, (LPARAM) def);
		def->ChangeFlags = cflg;
	} else {
		Tlen::Get()->GetFunction()->CallTS(TLEN_CONTACTLIST_CHANGEITEM, (WPARAM) id, (LPARAM) &item);
	}
}



void ContactList::AddIcon(int id, const ContactIcon& icon) {
	AddItemIconsDef def;
	InitializeStruct(def);
	def.ItemHandle = id;
	def.Icons = (ContactListIconDef*) &icon;
	def.IconsCount = 1;

	int ret = Tlen::Get()->GetFunction()->CallTS(TLEN_CONTACTLIST_ADDITEMICONS, (WPARAM) &def, NULL);

	if (!ret) {
		// wyjatek?
	}
}


void ContactList::DelIcon(int id, const char* icon_id) {

	int ret = Tlen::Get()->GetFunction()->CallTS(TLEN_CONTACTLIST_REDRAWLIST, (WPARAM) id, (LPARAM) icon_id);

	if (!ret) {
		// wyjatek?
	}
}


void ContactList::UpdateIcon(int id, const ContactIcon& icon) {

	int ret = Tlen::Get()->GetFunction()->CallTS(TLEN_CONTACTLIST_CHANGEITEMICON, (WPARAM) id, (LPARAM) &icon);

	if (!ret) {
		// wyjatek?
	}
}


const StringList* ContactList::GetSections() {
	ContactListItemsEnumerationDef def;
	InitializeStruct(def);
	def.EnumerationKind = CONTACTLIST_ENUMERATIONKIND_ALL;
	def.CallbackFunc = EnumCallbackSections;
	def.OwnerValue = (void*) &m_sections;
	Tlen::Get()->GetFunction()->CallTS(TLEN_CONTACTLIST_ENUMERATEITEMS, (WPARAM) &def, NULL);
	return &m_sections;
}


const StringList* ContactList::GetGroups(int section) {
	ContactListItemsEnumerationDef def;
	InitializeStruct(def);
	def.EnumerationKind = CONTACTLIST_ENUMERATIONKIND_CHILDREN;
	def.ParentHandle = section;		// tylko dzieci sekcji
	def.CallbackFunc = EnumCallbackGroups;
	def.OwnerValue = (void*) &m_groups;
	Tlen::Get()->GetFunction()->CallTS(TLEN_CONTACTLIST_ENUMERATEITEMS, (WPARAM) &def, NULL);
	return &m_groups;
}


int ContactList::EnumCallbackSections(WPARAM wParam, LPARAM lParam) {
	ContactListItemDef* item = (ContactListItemDef*) wParam;
	StringList* m_sections = (StringList*) lParam;

	if (!item) return 0;
	if (item->ItemKind != CONTACTLIST_ITEMKIND_SECTION) return 0;
	if (!item->MainText) return 0;

	m_sections->Add(item->MainText);
	return 0;
}


int ContactList::EnumCallbackGroups(WPARAM wParam, LPARAM lParam) {
	ContactListItemDef* item = (ContactListItemDef*) wParam;
	StringList* m_groups = (StringList*) lParam;

	if (!item) return 0;
	if (item->ItemKind != CONTACTLIST_ITEMKIND_GROUP) return 0;
	if (!item->MainText) return 0;

	m_groups->Add(item->MainText);
	return 0;
}


} // namespace tlenek
