/* ==========================================================================
 * Name:        event.cpp
 * Purpose:     Implementacja klas obslugi zdarzen
 * Author:      Marcin 'malcom' Malich <me@malcom.pl>
 * Modified by:
 * Created:     21/09/2007 23:42:32
 * SVN-ID:      $Id: event.cpp 21 2007-11-14 23:28:50Z malcom $
 * Copyright:   (c) 2007 Marcin 'malcom' Malich <me@malcom.pl>, Poland
 * Licence:     GNU Lesser General Public License
 * ========================================================================== */

#include <string.h>
#include "tlenek\event.h"
#include "tlenek\tlen.h"
#include "tlenek\dialoghandle.h"
#include "tlenek\optionswindow.h"


namespace tlenek {


// templates for event handlers //

int ContactIconEvt::Process() {
	switch (GetEventType()) {

		case CONTACTLIST_ITEMCALLBACK_MOUSEEXIT: {
			ItemCallBackMouseDef* data = (ItemCallBackMouseDef*) GetEventData();
			OnMouseExit(data->Position, data->MouseButton);
			break;
		}

		case CONTACTLIST_ITEMCALLBACK_MOUSEMOVE: {
			ItemCallBackMouseDef* data = (ItemCallBackMouseDef*) GetEventData();
			OnMouseMove(data->Position, data->MouseButton);
			break;
		}

		case CONTACTLIST_ITEMCALLBACK_MOUSEDOWN: {
			ItemCallBackMouseDef* data = (ItemCallBackMouseDef*) GetEventData();
			OnMouseDown(data->Position, data->MouseButton);
			break;
		}

		case CONTACTLIST_ITEMCALLBACK_MOUSEUP: {
			ItemCallBackMouseDef* data = (ItemCallBackMouseDef*) GetEventData();
			OnMouseUp(data->Position, data->MouseButton);
			break;
		}

		case CONTACTLIST_ITEMCALLBACK_MOUSEDBLCLK: {
			ItemCallBackMouseDef* data = (ItemCallBackMouseDef*) GetEventData();
			OnMouseDbClick(data->Position, data->MouseButton);
			break;
		}

	}
	return 0;
}


int ContactEvt::Process() {
	switch (GetEventType()) {

		case CONTACTLIST_ITEMCALLBACK_GETHEIGHT: {
			return OnGetHeight();
			break;
		}

		case CONTACTLIST_ITEMCALLBACK_DRAWITEM: {
			ItemCallBackOwnerDrawnDef* data = (ItemCallBackOwnerDrawnDef*) GetEventData();
			OnDraw(data->itemHdc, data->itemRect);
			break;
		}

		case CONTACTLIST_ITEMCALLBACK_CANSELECTITEM: {
			return OnSelect();
			break;
		}

		case CONTACTLIST_ITEMCALLBACK_MOUSEEXIT: {
			ItemCallBackMouseDef* data = (ItemCallBackMouseDef*) GetEventData();
			OnMouseExit(data->Position, data->MouseButton);
			break;
		}

		case CONTACTLIST_ITEMCALLBACK_MOUSEMOVE: {
			ItemCallBackMouseDef* data = (ItemCallBackMouseDef*) GetEventData();
			OnMouseMove(data->Position, data->MouseButton);
			break;
		}

		case CONTACTLIST_ITEMCALLBACK_MOUSEDOWN: {
			ItemCallBackMouseDef* data = (ItemCallBackMouseDef*) GetEventData();
			OnMouseDown(data->Position, data->MouseButton);
			break;
		}

		case CONTACTLIST_ITEMCALLBACK_MOUSEUP: {
			ItemCallBackMouseDef* data = (ItemCallBackMouseDef*) GetEventData();
			OnMouseUp(data->Position, data->MouseButton);
			break;
		}

		case CONTACTLIST_ITEMCALLBACK_MOUSEDBLCLK: {
			ItemCallBackMouseDef* data = (ItemCallBackMouseDef*) GetEventData();
			OnMouseDbClick(data->Position, data->MouseButton);
			break;
		}

		case CONTACTLIST_ITEMCALLBACK_KEYDOWN: {
			ItemCallBackKeyDef* data = (ItemCallBackKeyDef*) GetEventData();
			OnKeyDown(data->Key, data->KeyData);
			break;
		}

		case CONTACTLIST_ITEMCALLBACK_KEYUP: {
			ItemCallBackKeyDef* data = (ItemCallBackKeyDef*) GetEventData();
			OnKeyUp(data->Key, data->KeyData);
			break;
		}

	}
	return 0;
}


int OptionPageEvt::Process() {
	class OptionPage* page = GetOptionPage();
	HWND hwnd = page->GetHwnd();

	switch (GetEventType()) {

		case TLEN_OPTIONS_PAGE_ACTION_SHOW: {
			TlenOptionPageShowInfo* info = (TlenOptionPageShowInfo*) GetEventData();

			if (!IsWindow(hwnd)) {
				page->SetBgColor(info->bgColor);
				hwnd = OnCreate(info->Handle);
				if (!IsWindow(hwnd)) break;
				Tlen::Get()->GetDialogHandle()->Add(hwnd);
				page->SetHwnd(hwnd);
			}

			strncpy(info->Caption, page->GetTitle(), sizeof(info->Caption)-1);
			strncpy(info->Description, page->GetDescription(), sizeof(info->Description)-1);

			info->Icon = page->GetIcon();
			info->Flags = TLEN_OPTIONS_PAGEINFO_FLAG_ICONTRANS;

			SetWindowPos(hwnd, 0, info->x, info->y, info->width, info->height, SWP_NOZORDER);

			OnActionShow();

			ShowWindow(hwnd, SW_SHOW);
			break;
		}

		case TLEN_OPTIONS_PAGE_ACTION_HIDE: {

			if (IsWindow(hwnd)) {
				OnActionHide();
				ShowWindow(hwnd, SW_HIDE);
			}
			break;
		}

		case TLEN_OPTIONS_PAGE_ACTION_OK: {

			if (IsWindow(hwnd)) {
				OnClickOk();
				Tlen::Get()->GetDialogHandle()->Del(hwnd);
				OnDestroy(hwnd);
				page->SetHwnd(NULL);
			}
			break;
		}

		case TLEN_OPTIONS_PAGE_ACTION_APPLY: {

			if (IsWindow(hwnd)) {
				OnClickApply();
			}
			break;
		}

		case TLEN_OPTIONS_PAGE_ACTION_CANCEL: {

			if (IsWindow(hwnd)) {
				OnClickCancel();
				Tlen::Get()->GetDialogHandle()->Del(hwnd);
				OnDestroy(hwnd);
				page->SetHwnd(NULL);
			}
			break;
		}
	}
	return 0;
}



// templates for hook event handlers //

int ContactListEvt::Process() {
	switch (GetEventType()) {

		case BEFORE_ITEM_ADD: {
			return OnBeforeAdd((ContactItem*) GetEventData());
			break;
		}

		case AFTER_ITEM_ADD: {
			return OnAfterAdd((ContactItem*) GetEventData());
			break;
		}

		case BEFORE_ITEM_REMOVE: {
			return OnBeforeDel((ContactItem*) GetEventData());
			break;
		}

		case AFTER_ITEM_REMOVE: {
			return OnAfterDel((ContactItem*) GetEventData());
			break;
		}

		case BEFORE_ITEM_CHANGE: {
			ItemChangeDef* data = (ItemChangeDef*) GetEventData();
			return OnBeforeChange((ContactItem*) data->OldItem, (ContactItem*) data->NewItem);
			break;
		}

		case AFTER_ITEM_CHANGE: {
			ItemChangeDef* data = (ItemChangeDef*) GetEventData();
			return OnAfterChange((ContactItem*) data->OldItem, (ContactItem*) data->NewItem);
			break;
		}

	}
	return 0;
}


int ProtocolEvt::Process() {
	switch (GetEventType()) {

		case PROTOCOL_EVENT_BEFORERECVRAWDATA: {
			ProtocolRawDataDef* data = (ProtocolRawDataDef*) GetEventData();
			return OnBeforeRecvRawData(data->Data, data->DataLength);
			break;
		}

		case PROTOCOL_EVENT_AFTERRECVRAWDATA: {
			ProtocolRawDataDef* data = (ProtocolRawDataDef*) GetEventData();
			OnAfterRecvRawData(data->Data, data->DataLength);
			break;
		}

		case PROTOCOL_EVENT_AFTERPROCESSEDRECVRAWDATA: {
			ProtocolRawDataDef* data = (ProtocolRawDataDef*) GetEventData();
			OnAfterProcessedRecvRawData(data->Data, data->DataLength);
			break;
		}

		case PROTOCOL_EVENT_AFTERRECVMESSAGE: {
			ProtocolRawDataDef* data = (ProtocolRawDataDef*) GetEventData();
			OnAfterRecvMessage(data->Data, data->DataLength);
			break;
		}

		case PROTOCOL_EVENT_BEFORESENDRAWDATA: {
			ProtocolRawDataDef* data = (ProtocolRawDataDef*) GetEventData();
			return OnBeforeSendRawData(data->Data, data->DataLength);
			break;
		}

		case PROTOCOL_EVENT_AFTERSENDRAWDATA: {
			ProtocolRawDataDef* data = (ProtocolRawDataDef*) GetEventData();
			OnAfterSendRawData(data->Data, data->DataLength);
			break;
		}

		case PROTOCOL_EVENT_LOGGEDIN: {
			OnLoggedIn((ContactInfo*) GetEventData());
			break;
		}

		case PROTOCOL_EVENT_LOGGEDOUT: {
			OnLoggedOut((ContactInfo*) GetEventData());
			break;
		}

		case PROTOCOL_EVENT_RECVMESSAGE: {
			return OnRecvMessage((ProtocolMessage*) GetEventData());
			break;
		}

		case PROTOCOL_EVENT_BEFORESENDMESSAGE: {
			return OnBeforeSendMessage((ProtocolMessage*) GetEventData());
			break;
		}

		case PROTOCOL_EVENT_RECVPRESENCE: {
			return OnRecvPresence((ProtocolPresence*) GetEventData());
			break;
		}

		case PROTOCOL_EVENT_BEFORESTATUSCHANGE: {
			return OnBeforeSendPresence((ProtocolPresence*) GetEventData());
			break;
		}

		case PROTOCOL_EVENT_AUTHREQUEST: {
			OnAuthRequest((ProtocolAuthRequest*) GetEventData());
			break;
		}

		case PROTOCOL_EVENT_RECVROSTER: {
			OnRecvRoster();
			break;
		}

		case PROTOCOL_EVENT_CONTACTDELETED: {
			return OnContactDeleted((ContactInfo*) GetEventData());
			break;
		}
		case PROTOCOL_EVENT_CONTACTCHANGED: {
			ProtocolEventContactChanged* data = (ProtocolEventContactChanged*) GetEventData();
			return OnCcontactChanged((ContactInfo*) &data->old_ct, (ContactInfo*) &data->new_ct);
			break;
		}

	}
	return 0;
}


} // namespace tlenek
