/* ==========================================================================
 * Name:        hook.cpp
 * Purpose:     Implementacja klasy Hook
 * Author:      Marcin 'malcom' Malich <me@malcom.pl>
 * Modified by:
 * Created:     28/09/2007 13:34:04
 * SVN-ID:      $Id: hook.cpp 16 2007-10-17 20:40:54Z malcom $
 * Copyright:   (c) 2007 Marcin 'malcom' Malich <me@malcom.pl>, Poland
 * Licence:     GNU Lesser General Public License
 * ========================================================================== */

#include "tlenek\hook.h"
#include "tlenek\misc.h"
#include "tlenek\protocol.h"
#include "tlenek\tlen.h"
#include "tlenek\function.h"
#include "tlenek\exception.h"
#include "tlen_sdk\plugin_defs.h"
#include "tlen_sdk\plugin_struct.h"
#include "tlen_sdk\plugin_contactlist_defs.h"
#include "tlen_sdk\plugin_protocol_defs.h"
#include <list>
#include <iterator>


namespace tlenek {


int Hook::Attach(HookType type, TLENFUNCTION function, const char* param) {
	HANDLE handle;

	switch (type) {

		case APP_MESSAGE: {
			handle = GetTlenFunctions()->HookTlenEvent(GetPluginInstance(), TLEN_APPLICATION_MESSAGE_HOOK, function);
			break;
		}

		case PLUGINS_LOADED: {
			handle = GetTlenFunctions()->HookTlenEvent(GetPluginInstance(), TLEN_PLUGINS_LOADED_EVENT, function);
			break;
		}

		case CONTACTLIST: {
			handle = GetTlenFunctions()->HookTlenEvent(GetPluginInstance(), TLEN_CONTACTLIST_NOTIFICATION, function);
			break;
		}

		case ALL_PROTOCOL: {
			handle = GetTlenFunctions()->HookTlenEvent(GetPluginInstance(), TLEN_ALL_PROTOCOL_NOTIFICATION_HOOK, function);
			break;
		}

		case PROTOCOL: {
			ProtocolHookDef phd;
			InitializeStruct(phd);
			phd.ProtocolID = (char*) param;
			phd.CallBackFunc = function;
			phd.moduleHandle = GetPluginInstance();

			handle = (HANDLE) Tlen::Get()->GetFunction()->CallTS(TLEN_PROTOCOL_HOOK_EVENT, (WPARAM) &phd, NULL);
			break;
		}

	}

	en_t entry;
	entry.type = type;
	entry.handle = handle;
	entry.param = (char*) param;
	m_list.push_back(entry);

	return m_list.size();
}


void Hook::Detach(int id) {
	if (id > m_list.size()) {
		throw Exception(VALIDINDEX, NULL, "Hook::Detach", __LINE__, __FILE__);
	}

	en_list_t::iterator it = m_list.begin();
	std::advance(it, --id);

	detach(it);
}


void Hook::DetachAll(HookType type) {

	for (en_list_t::iterator it = m_list.begin(); it != m_list.end(); ++it) {
		if (it->type == type || type == ALL) {
			en_list_t::iterator pit = it;	// zapisujemy poprzedni iterator, aby nastepna
			it--;							// iteracja sprawdzila 'ta' pozycje, bo po jej
			detach(pit);					// usunieciu miejsce to zajmnie kolejny element
		}
	}

}


void Hook::detach(en_list_t::iterator& it) {

	switch (it->type) {

		case APP_MESSAGE:
		case PLUGINS_LOADED:
		case CONTACTLIST:
		case ALL_PROTOCOL: {
			GetTlenFunctions()->UnhookTlenEvent(GetPluginInstance(), it->handle);
			break;
		}

		case PROTOCOL: {
			Tlen::Get()->GetFunction()->CallTS(TLEN_PROTOCOL_UNHOOK_EVENT, (WPARAM) it->param, (LPARAM) it->handle);
			break;
		}

	}

	m_list.erase(it);
}


int Hook::Create(const char* name) {

	HANDLE handle = GetTlenFunctions()->CreateTlenEvent(GetPluginInstance(), name);
	m_h_list.push_back(handle);
	return (int) handle;

}


void Hook::Destroy(int id) {

	GetTlenFunctions()->DestroyTlenEvent(GetPluginInstance(), (HANDLE) id);
	m_h_list.remove((HANDLE) id);

}


void Hook::DestroyAll() {

	for (h_list_t::iterator it = m_h_list.begin(); it != m_h_list.end(); ++it) {
		GetTlenFunctions()->DestroyTlenEvent(GetPluginInstance(), *it);
	}
	m_h_list.clear();

}


void Hook::NotifyProto(const char* proto, ProtocolEventType event_type, void* even_data) {
	ProtocolEventDef ev;
	InitializeStruct(ev);
	ev.ProtocolID = (char*) proto;
	ev.EventID = event_type;
	Tlen::Get()->GetFunction()->CallTS(TLEN_PROTOCOL_NOTIFY_HOOKS, (WPARAM) &ev, (LPARAM) even_data);
}	


} // namespace tlenek
