/* ==========================================================================
 * Name:        inifile.cpp
 * Purpose:     Implementacja klasy IniFile
 * Author:      Marcin 'malcom' Malich <me@malcom.pl>
 * Modified by:
 * Created:     18/10/2007 10:55:54
 * SVN-ID:      $Id: inifile.cpp 17 2007-10-18 13:01:32Z malcom $
 * Copyright:   (c) 2007 Marcin 'malcom' Malich <me@malcom.pl>, Poland
 * Licence:     GNU Lesser General Public License
 * ========================================================================== */

#include <sstream>
#include "tlenek\inifile.h"
#include "tlenek\stringlist.h"
#include "tlenek\info.h"


namespace tlenek {


void IniFile::Load(const char* file) {
	if (file == NULL) {
		// standarodwe ladowanie pliku profile\PluginConfig\nazwa_pluga.ini
		std::stringstream ss;
		ss << info::GetProfilePath() << "PluginConfig\\" << info::GetPluginName() << ".ini";
		my_strdup(m_file, ss.str().c_str());
	} else {
		my_strdup(m_file, file);
	}
}


const char* IniFile::GetString(const char* section, const char* key, const char* def) {
	
	while (GetPrivateProfileString(section, key, def, m_buf, m_size, m_file) == m_size-1) {
		realloc_buf();
	}
	return m_buf;

}


StringList* IniFile::GetSection(const char* section) {
	int ret = GetPrivateProfileSection(section, m_buf, m_size, m_file);

	while (ret == m_size-2) {
		realloc_buf();
		ret = GetPrivateProfileSection(section, m_buf, m_size, m_file);
	}

	if (ret == 0) return NULL;

	m_sl.Clear();

	int i = 0;
	char* s = NULL;
	while (i != ret) {
		s = &m_buf[i];
		i += strlen(s)+1;
		m_sl.Add(s);
	}

	return &m_sl;
}


bool IniFile::SetSection(const char* section, const StringList& sl) {
	std::stringstream ss;
	for (int i=0; i<sl.Size(); i++) {
		ss << sl.Get(i) << '\0';
	}
	// przed zapisaniem musimy skasowac poprzednie dane
	WritePrivateProfileSection(section, NULL, m_file);
	return WritePrivateProfileSection(section, ss.str().c_str(), m_file) != 0;
}


StringList* IniFile::GetSectionNames() {
	int ret = GetPrivateProfileSectionNames(m_buf, m_size, m_file);

	while (ret == m_size-2) {
		realloc_buf();
		ret = GetPrivateProfileSectionNames(m_buf, m_size, m_file);
	}

	if (ret == 0) return NULL;

	m_sl.Clear();

	int i = 0;
	char* s = NULL;
	while (i != ret) {
		s = &m_buf[i];
		i += strlen(s)+1;
		m_sl.Add(s);
	}

	return &m_sl;
}

		
StringList* IniFile::GetKeyNames(const char* section) {
	int ret = GetPrivateProfileString(section, NULL, NULL, m_buf, m_size, m_file);

	while (ret == m_size-2) {
		realloc_buf();
		ret = GetPrivateProfileString(section, NULL, NULL, m_buf, m_size, m_file);
	}

	if (ret == 0) return NULL;

	m_sl.Clear();

	int i = 0;
	char* s = NULL;
	while (i != ret) {
		s = &m_buf[i];
		i += strlen(s)+1;
		m_sl.Add(s);
	}

	return &m_sl;
}


} // namespace tlenek
