/* ==========================================================================
 * Name:        protocols.cpp
 * Purpose:     Implementacje klas deklarowanych w naglowku tlenek\protocols.h
 * Author:      Marcin 'malcom' Malich <me@malcom.pl>
 * Modified by:
 * Created:     11/10/2007 22:39:53
 * SVN-ID:      $Id: protocols.cpp 12 2007-10-12 01:02:51Z malcom $
 * Copyright:   (c) 2007 Marcin 'malcom' Malich <me@malcom.pl>, Poland
 * Licence:     GNU Lesser General Public License
 * ========================================================================== */

#include "tlenek\protocols.h"
#include "tlenek\contactinfo.h"
#include "tlenek\utils.h"


namespace tlenek {


ProtocolMessage::ProtocolMessage(const ProtocolMessage& msg) {
	structSize = msg.structSize;
	CopyContactDef(&Contact, &msg.Contact);
	CopyContactsListDef(&MultipleContacts, &msg.MultipleContacts);
	Flags = msg.Flags;
	TimeStamp = msg.TimeStamp;
	my_cstrdup(MessageID, msg.MessageID);
	my_cstrdup(MessageText, msg.MessageText);
	my_cstrdup(RoomName, msg.RoomName);
	my_cstrdup(UserName, msg.UserName);
	my_cstrdup(UserEmail, msg.UserEmail);
	my_cstrdup(WWWGate, msg.WWWGate);
	MessageHandle = msg.MessageHandle;
	OwnerData = msg.OwnerData;
}


ProtocolMessage::~ProtocolMessage() {
	FreeContactDef(&Contact);
	FreeContactsListDef(&MultipleContacts);
	free(MessageID);
	free(MessageText);
	free(RoomName);
	free(UserName);
	free(UserEmail);
	free(WWWGate);
}



ProtocolRawData::ProtocolRawData(const ProtocolRawData& rawdata) {
	structSize = rawdata.structSize;
	DataLength = rawdata.DataLength;
	Data = rawdata.Data;
	OwnerData = rawdata.OwnerData;
	Flags = rawdata.Flags;
}



ProtocolAuthRequest::ProtocolAuthRequest(const ProtocolAuthRequest& request) {
	structSize = request.structSize;
	CopyContactDef(&Contact, &request.Contact);
	Flags = request.Flags;
	my_cstrdup(Reason, request.Reason);
	TimeStamp = request.TimeStamp;
	OwnerData = request.OwnerData;
}



ProtocolPresence::ProtocolPresence(const ProtocolPresence& presence) {
	structSize = presence.structSize;
	CopyContactDef(&Contact, &presence.Contact);
	CurrentState = presence.CurrentState;
	PreviousState = presence.PreviousState;
	my_cstrdup(CurrentDescription, presence.CurrentDescription);
	my_cstrdup(PreviousDescription, presence.PreviousDescription);
	Flags = presence.Flags;
	TimeStamp = presence.TimeStamp;
	CurrentComeBackTimeStamp = presence.CurrentComeBackTimeStamp;
	PreviousComeBackTimeStamp = presence.PreviousComeBackTimeStamp;
	OwnerData = presence.OwnerData;
}


} // namespace tlenek
