/* ==========================================================================
 * Name:        contactinfo.cpp
 * Purpose:     Definicja klas StringList
 * Author:      Marcin 'malcom' Malich <me@malcom.pl>
 * Modified by:
 * Created:     09/10/2007 12:41:24
 * SVN-ID:      $Id: stringlist.cpp 10 2007-10-10 19:03:31Z malcom $
 * Copyright:   (c) 2007 Marcin 'malcom' Malich <me@malcom.pl>, Poland
 * Licence:     GNU Lesser General Public License
 * ========================================================================== */

#include "tlenek\stringlist.h"
#include "tlenek\tlen.h"
#include "tlenek\function.h"
#include "tlenek\exception.h"


namespace tlenek {


bool StringList::proc = false;

sl_init_type       StringList::sl_init       = NULL;
sl_add_type        StringList::sl_add        = NULL;
sl_delete_type     StringList::sl_delete     = NULL;
sl_get_type        StringList::sl_get        = NULL;
sl_find_type       StringList::sl_find       = NULL;
sl_free_type       StringList::sl_free       = NULL;
sl_addstrings_type StringList::sl_addstrings = NULL;
sl_commonpart_type StringList::sl_commonpart = NULL;


void StringList::Init() {
	if (!proc) {
		sl_init       = (sl_init_type)       Tlen::Get()->GetFunction()->GetProcAddr(TPA_SL_INIT);
		sl_add        = (sl_add_type)        Tlen::Get()->GetFunction()->GetProcAddr(TPA_SL_ADD);
		sl_delete     = (sl_delete_type)     Tlen::Get()->GetFunction()->GetProcAddr(TPA_SL_DELETE);
		sl_get        = (sl_get_type)        Tlen::Get()->GetFunction()->GetProcAddr(TPA_SL_GET);
		sl_find       = (sl_find_type)       Tlen::Get()->GetFunction()->GetProcAddr(TPA_SL_FIND);
		sl_free       = (sl_free_type)       Tlen::Get()->GetFunction()->GetProcAddr(TPA_SL_FREE);
		sl_addstrings = (sl_addstrings_type) Tlen::Get()->GetFunction()->GetProcAddr(TPA_SL_ADDSTRINGS);
		sl_commonpart = (sl_commonpart_type) Tlen::Get()->GetFunction()->GetProcAddr(TPA_SL_COMMONPART);
		proc = true;
	}
	sl_init(this);
}


void StringList::Del(int index) {
	if (index >= 0 && index < count) {
		sl_delete(this, index);
	} else {
		throw Exception(VALIDINDEX, NULL, "StringList::Del", __LINE__, __FILE__);
	}
}


void StringList::Del(int start, int end) {
	if (start >= 0 && start < count && end >= 0 && end < count) {
		// usuwamy od konca, wtedy bedzie mniej danych do przenoszenia w pamieci
		start--;	// start musi usunac
		end--;		// a end musi pozostac
		for (int i=end; i>start; i--) {
			sl_delete(this, i);
		}
	} else {
		throw Exception(VALIDINDEX, NULL, "StringList::Del", __LINE__, __FILE__);
	}
}


void StringList::Del(const char* string) {
	int idx = Find(string);

	while (idx > -1) {
		Del(idx);
		idx = Find(string);
	}

}
	

const char* StringList::Get(int index) const {
	if (index >= 0 && index < count) {
		return sl_get((StringListDef*) this, index);
	} else {
		throw Exception(VALIDINDEX, NULL, "StringList::Get", __LINE__, __FILE__);
	}
}


} // namespace tlenek
