/* ==========================================================================
 * Name:        xml.cpp
 * Purpose:     Implementacja klas parsera XML wykorzystujacego libexpat.dll
 * Author:      Marcin 'malcom' Malich <me@malcom.pl>
 * Modified by:
 * Created:     16/11/2007  18:45:11
 * SVN-ID:      $Id: xml.cpp 23 2007-11-19 22:05:03Z malcom $
 * Copyright:   (c) 2007 Marcin 'malcom' Malich <me@malcom.pl>, Poland
 * Licence:     GNU Lesser General Public License
 * ========================================================================== */

#include "tlenek\xml.h"
#include "tlenek\utils.h"
#include <iostream>
#include <fstream>
#include <sstream>
#include <string>
#include <string.h>


namespace tlenek {
namespace xml {

using std::ostream;
using std::istream;


XML_ErrorString_t XML_ErrorString;
XML_GetCurrentLineNumber_t XML_GetCurrentLineNumber;
XML_GetErrorCode_t XML_GetErrorCode;
XML_Parse_t XML_Parse;
XML_ParserCreate_t XML_ParserCreate;
XML_ParserFree_t XML_ParserFree;
XML_SetCharacterDataHandler_t XML_SetCharacterDataHandler;
XML_SetCommentHandler_t XML_SetCommentHandler;
XML_SetDefaultHandler_t XML_SetDefaultHandler;
XML_SetElementHandler_t XML_SetElementHandler;
XML_SetStartCdataSectionHandler_t XML_SetStartCdataSectionHandler;
XML_SetUnknownEncodingHandler_t XML_SetUnknownEncodingHandler;
XML_SetUserData_t XML_SetUserData;


void Init(const char* dir) {
	char* path = new char[strlen(dir)+18];
	strcpy(path, dir);
	strcat(path, "libs\\libexpat.dll");


	hModule = LoadLibrary(path);
	if (hModule == NULL) {
		// wyjatek
	}

	delete [] path;

	// pobieramy adresy funkcji

	XML_ErrorString = (XML_ErrorString_t) GetProcAddress(hModule, "XML_ErrorString");
	if (XML_ErrorString == NULL) {
		// wyjatek
	}

	XML_GetCurrentLineNumber = (XML_GetCurrentLineNumber_t) GetProcAddress(hModule, "XML_GetCurrentLineNumber");
	if (XML_GetCurrentLineNumber == NULL) {
		// wyjatek
	}

	XML_Parse = (XML_Parse_t) GetProcAddress(hModule, "XML_Parse");
	if (XML_Parse == NULL) {
		// wyjatek
	}

	XML_ParserFree = (XML_ParserFree_t) GetProcAddress(hModule, "XML_ParserFree");
	if (XML_ParserFree == NULL) {
		// wyjatek
	}

	XML_GetErrorCode = (XML_GetErrorCode_t) GetProcAddress(hModule, "XML_GetErrorCode");
	if (XML_GetErrorCode == NULL) {
		// wyjatek
	}

	XML_ParserCreate = (XML_ParserCreate_t) GetProcAddress(hModule, "XML_ParserCreate");
	if (XML_ParserCreate == NULL) {
		// wyjatek
	}

	XML_SetCharacterDataHandler = (XML_SetCharacterDataHandler_t) GetProcAddress(hModule, "XML_SetCharacterDataHandler");
	if (XML_SetCharacterDataHandler == NULL) {
		// wyjatek
	}

	XML_SetCommentHandler = (XML_SetCommentHandler_t) GetProcAddress(hModule, "XML_SetCommentHandler");
	if (XML_SetCommentHandler == NULL) {
		// wyjatek
	}

	XML_SetDefaultHandler = (XML_SetDefaultHandler_t) GetProcAddress(hModule, "XML_SetDefaultHandler");
	if (XML_SetDefaultHandler == NULL) {
		// wyjatek
	}

	XML_SetElementHandler = (XML_SetElementHandler_t) GetProcAddress(hModule, "XML_SetElementHandler");
	if (XML_SetElementHandler == NULL) {
		// wyjatek
	}

	XML_SetStartCdataSectionHandler = (XML_SetStartCdataSectionHandler_t) GetProcAddress(hModule, "XML_SetStartCdataSectionHandler");
	if (XML_SetStartCdataSectionHandler == NULL) {
		// wyjatek
	}

	XML_SetUnknownEncodingHandler = (XML_SetUnknownEncodingHandler_t) GetProcAddress(hModule, "XML_SetUnknownEncodingHandler");
	if (XML_SetUnknownEncodingHandler == NULL) {
		// wyjatek
	}

	XML_SetUserData = (XML_SetUserData_t) GetProcAddress(hModule, "XML_SetUserData");
	if (XML_SetUserData == NULL) {
		// wyjatek
	}
}


/*  Some of the code adapted from wxXml */

//-----------------------------------------------------------------------------
//  Node
//-----------------------------------------------------------------------------

void Node::DoCopy(const Node& node) {
	m_type = node.m_type;
	m_name = node.m_name;
	m_content = node.m_content;
	m_lineNo = node.m_lineNo;
	m_children = NULL;

	Node* n = node.m_children;
	while (n) {
		AddChild(new Node(*n));
		n = n->GetNext();
	}

	m_attrs = NULL;

	Attribute* a = node.m_attrs;
    while (a) {
		AddAttribute(a->GetName(), a->GetValue());
		a = a->GetNext();
	}
}


Node::Node(Node* parent, NodeType type, const char* name, const char* content,
	 Attribute* attrs, Node* next, int lineNo) {
    
	m_type = type;
	my_strdup(m_name, name, false);
	my_strdup(m_content, content, false);
	m_attrs = attrs;
	m_parent = parent;
	m_children = NULL;
	m_next = next;
	m_lineNo = lineNo;

	if (m_parent) {
		if (m_parent->m_children) {
			m_next = m_parent->m_children;
			m_parent->m_children = this;
		} else {
			m_parent->m_children = this;
		}
	}
}


Node::Node(NodeType type, const char* name, const char* content, int lineNo) {
	m_type = type;
	my_strdup(m_name, name, false);
	my_strdup(m_content, content, false);
	m_attrs = NULL;
	m_parent = NULL;
	m_children = NULL;
	m_next = NULL;
	m_lineNo = lineNo;
}


Node::Node(const Node& node) {
	m_next = NULL;
	m_parent = NULL;
	DoCopy(node);
}


Node::~Node() {
	delete [] m_name;
	delete [] m_content;

	Node *c, *c2;
	for (c = m_children; c; c = c2) {
		c2 = c->m_next;
		delete c;
	}

	Attribute *a, *a2;
	for (a = m_attrs; a; a = a2) {
		a2 = a->GetNext();
		delete a;
	}
}


Node& Node::operator=(const Node& node) {
	delete m_attrs;
	delete m_children;
	DoCopy(node);
	return *this;
}


void Node::AddChild(Node* child) {

	if (m_children == NULL) {
		m_children = child;

	} else {
		Node* ch = m_children;
		while (ch->m_next)
			ch = ch->m_next;
		ch->m_next = child;
	}

	child->m_next = NULL;
	child->m_parent = this;
}


bool Node::InsertChild(Node* child, Node* before_node) {
	if (child == NULL)
		return false;

	if (m_children == before_node) {
		m_children = child;

	} else if (m_children == NULL) {
		if (before_node != NULL) {
			return false;		// we have no children so we don't need to search
		}
		m_children = child;

	} else if (before_node == NULL) {
		// prepend child
		child->m_parent = this;
		child->m_next = m_children;
		m_children = child;
		return true;

	} else {
		Node* ch = m_children;
		while (ch && ch->m_next != before_node)
			ch = ch->m_next;

		if (!ch)
			return false;		// before_node not found

		ch->m_next = child;
	}

	child->m_parent = this;
	child->m_next = before_node;
	return true;
}


bool Node::RemoveChild(Node* child) {

	if (m_children == NULL) {
		return false;

	} else if (m_children == child) {
		m_children = child->m_next;
		child->m_parent = NULL;
		child->m_next = NULL;
		return true;

	} else {
		Node* ch = m_children;
		while (ch->m_next) {
			if (ch->m_next == child) {
				ch->m_next = child->m_next;
				child->m_parent = NULL;
				child->m_next = NULL;
				return true;
			}
			ch = ch->m_next;
		}
		return false;
	}
}


void Node::AddAttribute(Attribute* attr) {

	if (m_attrs == NULL) {
		m_attrs = attr;

	} else {
		Attribute* a = m_attrs;
		while (a->GetNext())
			a = a->GetNext();
		a->SetNext(attr);
	}
}


bool Node::GetAttribute(const char* name, char*& value) const {
	if (value == NULL)
		return false;

	Attribute* attr = GetAttributes();
	while (attr) {
		if (strcmp(attr->GetName(), name) == 0) {
			value = (char*) attr->GetValue();
			return true;
		}
		attr = attr->GetNext();
	}

	return false;
}


const char* Node::GetAttribute(const char* name, const char* defaultVal) const {
	char* tmp;

	if (GetAttribute(name, tmp))
		return tmp;

	return defaultVal;
}


bool Node::DeleteAttribute(const char* name) {
	Attribute* attr;

	if (m_attrs == NULL) {
		return false;

	} else if (m_attrs->GetName() == name) {
		attr = m_attrs;
		m_attrs = attr->GetNext();
		attr->SetNext(NULL);
		delete attr;
		return true;

	} else {
		Attribute* a = m_attrs;
		while (a->GetNext()) {
			if (a->GetNext()->GetName() == name) {
				attr = a->GetNext();
				a->SetNext(attr->GetNext());
				attr->SetNext(NULL);
				delete attr;
				return true;
			}
			a = a->GetNext();
		}
		return false;
	}
}


const char* Node::GetNodeContent() const {
	Node* n = GetChildren();

	while (n) {
		if (n->GetType() == TEXT_NODE || n->GetType() == CDATA_SECTION_NODE)
			return n->GetContent();
		n = n->GetNext();
	}

	return NULL;
}


int Node::GetDepth(Node* grandparent) const {
	const Node* n = this;
	int ret = -1;

	do {
		ret++;
		n = n->GetParent();
		if (n == grandparent)
			return ret;
	} while (n);

	return -1;
}



//-----------------------------------------------------------------------------
//  Document
//-----------------------------------------------------------------------------

void Document::DoCopy(const Document& doc) {
	my_strdup(m_version, doc.m_version, false);
	my_strdup(m_encoding, doc.m_encoding, false);

	if (doc.m_root) {
		m_root = new Node(*doc.m_root);
	} else {
		m_root = NULL;
	}
}


Document::Document() {
	Init();
	m_root = NULL;
	my_strdup(m_version, "1.0", false);
	my_strdup(m_encoding, "UTF-8", false);
}


Document::Document(const char* filename, const char* encoding) {
	Init();
	m_root = NULL;

	if (!Load(filename, encoding)) {
		delete m_root;
		m_root = NULL;
	}
}


Document::Document(const Document& doc) {
	DoCopy(doc);
}


Document& Document::operator=(const Document& doc) {
	delete m_root;
	DoCopy(doc);
	return *this;
}


bool Document::Load(const char* filename, const char* encoding, bool keep_wnodes) {
	std::ifstream stream;
	stream.open(filename);
	if (!stream)
		return false;
	return Load(stream, encoding, keep_wnodes);
}


bool Document::Save(const char* filename, int indentstep) const {
	std::ofstream stream;
	stream.open(filename);
	if (!stream)
		return false;
	return Save(stream, indentstep);
}


bool Document::FromString(const char* str, size_t len) {
	if (str == NULL)
		return false;
	std::istringstream stream;
	stream.str(std::string(str, len));
	return Load(stream);
}


bool Document::ToString(char* buf, size_t* len) {
	if (len == NULL)
		return false;
	std::ostringstream  stream;
	if (!Save(stream))
		return false;
	std::string str = stream.str();
	if (buf == NULL || str.length() > *len) {
		*len = str.length();
		return false;
	}
	*len = str.copy(buf, str.length());
	return true;
}


//-----------------------------------------------------------------------------
//  Document loading routines
//-----------------------------------------------------------------------------

struct XmlParsingContext {
	XML_Parser parser;
	Node* root;
	Node* node;
	Node* lastAsText;
	std::string encoding;
	std::string version;
	bool removeWhiteOnlyNodes;
};


extern "C" {

static void StartElementHnd(void* userData, const char* name, const char** atts) {
	XmlParsingContext* ctx = (XmlParsingContext*) userData;
	Node* node = new Node(ELEMENT_NODE, name, NULL, XML_GetCurrentLineNumber(ctx->parser));

	const char** a = atts;
	while (*a) {
		node->AddAttribute(a[0], a[1]);
		a += 2;
	}

	if (ctx->root == NULL) {
		ctx->root = node;
	} else {
		ctx->node->AddChild(node);
	}
	ctx->node = node;
	ctx->lastAsText = NULL;
}


static void EndElementHnd(void* userData, const char* name) {
	XmlParsingContext* ctx = (XmlParsingContext*) userData;
	ctx->node = ctx->node->GetParent();
	ctx->lastAsText = NULL;
}


static void TextHnd(void* userData, const char* s, int len) {
	XmlParsingContext* ctx = (XmlParsingContext*) userData;

	if (ctx->lastAsText) {
		std::string buf(ctx->lastAsText->GetContent());
		buf += s;
		ctx->lastAsText->SetContent(buf.c_str());

	} else {
		bool whiteOnly = false;
		if (ctx->removeWhiteOnlyNodes)
			whiteOnly = StrIsWhiteOnly(s);

		if (!whiteOnly) {
			ctx->lastAsText = new Node(TEXT_NODE, "text", s, XML_GetCurrentLineNumber(ctx->parser));
			ctx->node->AddChild(ctx->lastAsText);
		}
	}
}


static void StartCdataHnd(void* userData) {
	XmlParsingContext* ctx = (XmlParsingContext*) userData;
	ctx->lastAsText = new Node(CDATA_SECTION_NODE, "cdata", "", XML_GetCurrentLineNumber(ctx->parser));
	ctx->node->AddChild(ctx->lastAsText);
}


static void CommentHnd(void* userData, const char* data) {
	XmlParsingContext* ctx = (XmlParsingContext*) userData;

	if (ctx->node) {
		// VS: ctx->node == NULL happens if there is a comment before
		//     the root element (e.g. wxDesigner's output). We ignore such
		//     comments, no big deal...
		ctx->node->AddChild(new Node(COMMENT_NODE, "comment", data, XML_GetCurrentLineNumber(ctx->parser)));
	}
	ctx->lastAsText = NULL;
}


static void DefaultHnd(void* userData, const char* s, int len) {
	// XML header:
	if (len > 6 && strncmp(s, "<?xml ", 6) == 0) {
		XmlParsingContext* ctx = (XmlParsingContext*) userData;

		std::string buf(s, len);
		std::string::size_type p1, p2;

		p1 = buf.find("encoding=");
		if (p1 != std::string::npos) {
			p2 = buf.find(buf[p1+9], p1 + 10);
			ctx->encoding = buf.substr(p1 + 10, p2 - (p1 + 10));
		}

		p1 = buf.find("version=");
		if (p1 != std::string::npos) {
			p2 = buf.find(buf[p1+8], p1 + 9);
			ctx->version = buf.substr(p1 + 9, p2 - (p1 + 9));
		}
	}
}


static int UnknownEncodingHnd(void* encodingHandlerData, const XML_Char* name, XML_Encoding* info) {
	// We must build conversion table for expat.

	return 1;
}

} // extern "C"


bool Document::Load(istream& stream, const char* encoding, bool keep_wnodes) {
	XmlParsingContext ctx;
	XML_Parser parser = XML_ParserCreate(NULL);

	ctx.root = ctx.node = NULL;
	ctx.encoding = encoding;		// default in absence of encoding=""
	ctx.removeWhiteOnlyNodes = keep_wnodes;
	ctx.parser = parser;

	XML_SetUserData(parser, (void*) &ctx);
	XML_SetElementHandler(parser, StartElementHnd, EndElementHnd);
	XML_SetCharacterDataHandler(parser, TextHnd);
	XML_SetStartCdataSectionHandler(parser, StartCdataHnd);
	XML_SetCommentHandler(parser, CommentHnd);
	XML_SetDefaultHandler(parser, DefaultHnd);
//	XML_SetUnknownEncodingHandler(parser, UnknownEncodingHnd, NULL);

	bool ok = true;
	bool done;
	char buf[1024];

	do {
		stream.read(buf, sizeof(buf));
		size_t len = stream.gcount();
		done = len < sizeof(buf);
		if (!XML_Parse(parser, buf, len, done)) {
			// wyjatek
			// "XML parsing error: '%s' at line %d", 
			// XML_ErrorString(XML_GetErrorCode(parser)), XML_GetCurrentLineNumber(parser))
			ok = false;
			break;
		}
	} while (!done);

	if (ok) {
		if (!ctx.version.empty())
			SetVersion(ctx.version.c_str());
		if (!ctx.encoding.empty())
			SetEncoding(ctx.encoding.c_str());
		SetRoot(ctx.root);

	} else {
		delete ctx.root;
	}

	XML_ParserFree(parser);

	return ok;
}


//-----------------------------------------------------------------------------
//  Document saving routines
//-----------------------------------------------------------------------------

// Same as above, but create entities first.
// Translates '<' to "&lt;", '>' to "&gt;" and '&' to "&amp;"
static void OutputStringEnt(ostream& stream, const char* str, bool escape_quotes = false) {
	char c;
	while (c = *str++) {
		if (c == '<') {
			stream << "&lt;";
		} else if (c == '>') {
			stream << "&gt;";
		} else if (c == '&' && strncmp(str, "amp;", 4) != 0) {
			stream << "&amp;";
		} else if (c == '"' && escape_quotes) {
			stream << "&quot;";
		} else {
			stream << c;
		}
	}
}


inline static void OutputIndentation(ostream& stream, int indent) {
	stream << "\n";
	for (int i = 0; i < indent; i++)
		stream << "\t";
}


static void OutputNode(ostream& stream, Node* node, int indent, int indentstep) {
	Node *n, *prev;
	Attribute *attr;

	switch (node->GetType()) {
		case CDATA_SECTION_NODE:
			stream << "<![CDATA[" << node->GetContent() << "]]>";
			break;

		case TEXT_NODE:
			OutputStringEnt(stream, node->GetContent());
			break;

		case ELEMENT_NODE:
			stream << "<" << node->GetName();

			attr = node->GetAttributes();
			while (attr) {
				stream << " " << attr->GetName() <<  "=\"";
				OutputStringEnt(stream, attr->GetValue(), true);
				stream << "\"";
				attr = attr->GetNext();
			}

			if (node->GetChildren()) {
				stream << ">";
				prev = NULL;
				n = node->GetChildren();
				while (n) {
					if (indentstep >= 0 && n && n->GetType() != TEXT_NODE)
						OutputIndentation(stream, indent + indentstep);
					OutputNode(stream, n, indent + indentstep, indentstep);
					prev = n;
					n = n->GetNext();
				}
				if (indentstep >= 0 && prev && prev->GetType() != TEXT_NODE)
					OutputIndentation(stream, indent);
				stream << "</" << node->GetName() << ">";
			} else {
				stream << "/>";
			}
			break;

		case COMMENT_NODE:
			stream << "<!--" << node->GetContent() << "-->";
			break;

//		default:
//			// wyjatek // unsupported node type
	}
}


bool Document::Save(ostream& stream, int indentstep) const {

	if (!IsOk())
		return false;

	stream << "<?xml version=\"" << GetVersion() << "\" encoding=\"" << GetEncoding() << "\"?>\n";
	OutputNode(stream, GetRoot(), 0, indentstep);
	stream << "\n";

	return true;
}


} // namespace xml
} // namespace